"""Template-based generator for language plugins and tests.

This module implements the TemplateGeneratorContract to generate language plugin
files and test files from Jinja2 templates.
"""

from __future__ import annotations

import ast
import re
from pathlib import Path

from jinja2 import Environment, FileSystemLoader, StrictUndefined

from .contracts.template_generator_contract import TemplateGeneratorContract


class TemplateGenerator(TemplateGeneratorContract):
    """Generate language plugin and test files from templates."""

    def __init__(self, template_dir: Path | None = None):
        """Initialize the template generator.

        Args:
            template_dir: Directory containing templates. Defaults to templates/ in package.
        """
        if template_dir is None:
            template_dir = Path(__file__).parent.parent / "templates"
        self.template_dir = template_dir
        self._env = Environment(
            loader=FileSystemLoader(str(template_dir)),
            undefined=StrictUndefined,
            trim_blocks=True,
            lstrip_blocks=True,
            autoescape=True,
        )

    def generate_plugin(
        self,
        language_name: str,
        config: dict[str, any],
    ) -> tuple[bool, Path]:
        """Generate a language plugin file from template.

        Args:
            language_name: Name of the language (e.g., 'css', 'html')
            config: Configuration including node types, file extensions, etc.

        Returns:
            Tuple of (success, path to generated file)
        """
        if not self._validate_language_name(language_name):
            return False, Path()
        if not self._validate_config(config):
            return False, Path()
        template_vars = self._prepare_plugin_variables(language_name, config)
        try:
            template = self._env.get_template("language_plugin.py.j2")
            content = template.render(**template_vars)
        except (FileNotFoundError, OSError, ValueError) as e:
            print(f"Error rendering template: {e}")
            return False, Path()
        output_path = (
            Path(
                __file__,
            ).parent
            / "languages"
            / f"{language_name}.py"
        )
        try:
            output_path.parent.mkdir(exist_ok=True)
            output_path.write_text(content, encoding="utf-8")
        except (FileNotFoundError, OSError, ValueError) as e:
            print(f"Error writing file: {e}")
            return False, Path()
        return True, output_path

    def generate_test(
        self,
        language_name: str,
        test_cases: list[dict[str, str]],
    ) -> tuple[bool, Path]:
        """Generate test file for a language plugin.

        Args:
            language_name: Name of the language
            test_cases: List of test case definitions

        Returns:
            Tuple of (success, path to generated test file)
        """
        if not self._validate_language_name(language_name):
            return False, Path()
        if not self._validate_test_cases(test_cases):
            return False, Path()
        plugin_path = (
            Path(
                __file__,
            ).parent
            / "languages"
            / f"{language_name}.py"
        )
        if not plugin_path.exists():
            print(f"Plugin for {language_name} does not exist")
            return False, Path()
        template_vars = self._prepare_test_variables(language_name, test_cases)
        try:
            template = self._env.get_template("language_test.py.j2")
            content = template.render(**template_vars)
        except (FileNotFoundError, OSError, ValueError) as e:
            print(f"Error rendering template: {e}")
            return False, Path()
        output_path = (
            Path(
                __file__,
            ).parent.parent
            / "tests"
            / f"test_{language_name}_language.py"
        )
        try:
            output_path.parent.mkdir(exist_ok=True)
            output_path.write_text(content, encoding="utf-8")
        except (FileNotFoundError, OSError, ValueError) as e:
            print(f"Error writing file: {e}")
            return False, Path()
        return True, output_path

    @staticmethod
    def validate_plugin(plugin_path: Path) -> tuple[bool, list[str]]:
        """Validate a generated plugin file.

        Args:
            plugin_path: Path to plugin file

        Returns:
            Tuple of (is_valid, list of issues if any)
        """
        issues = []
        if not plugin_path.exists():
            return False, ["Plugin file does not exist"]
        if not plugin_path.is_file():
            return False, ["Plugin path is not a file"]
        try:
            content = plugin_path.read_text(encoding="utf-8")
        except (FileNotFoundError, OSError) as e:
            return False, [f"Could not read file: {e}"]
        try:
            ast.parse(content)
        except SyntaxError as e:
            issues.append(f"Syntax error: {e}")
        required_imports = [
            "from tree_sitter import Node",
            "from .plugin_base import LanguagePlugin",
            "from ..contracts.language_plugin_contract import ExtendedLanguagePluginContract",
        ]
        issues.extend(
            f"Missing required import: {imp}"
            for imp in required_imports
            if imp not in content
        )
        class_pattern = (
            "class \\w+Plugin\\(.*LanguagePlugin.*ExtendedLanguagePluginContract.*\\):"
        )
        if not re.search(class_pattern, content):
            issues.append(
                "Plugin class must inherit from both LanguagePlugin and ExtendedLanguagePluginContract",
            )
        required_methods = [
            "get_semantic_chunks",
            "get_chunk_node_types",
            "should_chunk_node",
            "get_node_context",
            "language_name",
            "supported_extensions",
            "default_chunk_types",
            "get_node_name",
        ]
        issues.extend(
            f"Missing required method: {method}"
            for method in required_methods
            if f"def {method}" not in content and f"def {method}(" not in content
        )
        return len(issues) == 0, issues

    @staticmethod
    def _validate_language_name(name: str) -> bool:
        """Validate language name is lowercase alphanumeric."""
        return bool(re.match(r"^[a-z0-9]+$", name))

    @staticmethod
    def _validate_config(config: dict[str, any]) -> bool:
        """Validate config has required fields."""
        return "node_types" in config and "file_extensions" in config

    @staticmethod
    def _validate_test_cases(test_cases: list[dict[str, str]]) -> bool:
        """Validate test cases have required fields."""
        if not test_cases:
            return False
        for case in test_cases:
            if "name" not in case or "code" not in case:
                return False
        return True

    @staticmethod
    def _prepare_plugin_variables(
        language_name: str,
        config: dict[str, any],
    ) -> dict[str, any]:
        """Prepare variables for plugin template rendering."""
        class_name = language_name.capitalize()
        file_extensions = [
            (ext if ext.startswith(".") else f".{ext}")
            for ext in config["file_extensions"]
        ]
        return {
            "language_name": language_name,
            "class_name": class_name,
            "node_types": config["node_types"],
            "file_extensions": file_extensions,
            "include_imports": config.get("include_imports", True),
            "include_decorators": config.get("include_decorators", False),
            "include_nested": config.get("include_nested", True),
            "custom_node_handling": config.get("custom_node_handling", {}),
        }

    @staticmethod
    def _prepare_test_variables(
        language_name: str,
        test_cases: list[dict[str, str]],
    ) -> dict[str, any]:
        """Prepare variables for test template rendering."""
        class_name = language_name.capitalize()
        processed_cases = [
            {
                "name": case["name"],
                "code": case["code"],
                "expected_chunks": case.get("expected_chunks", 1),
                "expected_types": case.get("expected_types", []),
            }
            for case in test_cases
        ]
        return {
            "language_name": language_name,
            "class_name": class_name,
            "test_cases": processed_cases,
        }



"""Test suite for Phase 1.5: Extended language support for call span extraction."""

from typing import Any

import pytest
from tree_sitter import Node

from chunker.metadata import MetadataExtractorFactory
from chunker.metadata.extractor import BaseMetadataExtractor
from chunker.parser import get_parser


class SimpleMetadataExtractor(BaseMetadataExtractor):
    """Simple concrete implementation for testing."""

    def extract_imports(self, node: Node, source: bytes) -> list[str]:
        """Extract imports (not needed for call testing)."""
        return []

    def extract_exports(self, node: Node, source: bytes) -> list[str]:
        """Extract exports (not needed for call testing)."""
        return []

    def extract_dependencies(self, node: Node, source: bytes) -> list[str]:
        """Extract dependencies (not needed for call testing)."""
        return []

    def extract_signature(self, node: Node, source: bytes) -> dict[str, Any] | None:
        """Extract signature (not needed for call testing)."""
        return None

    def extract_docstring(self, node: Node, source: bytes) -> str | None:
        """Extract docstring (not needed for call testing)."""
        return None


class TestPhase15Languages:
    """Test call span extraction for 15+ languages."""

    @staticmethod
    def _test_language_calls(language: str, code: str, expected_calls: list[str]):
        """Helper to test call extraction for a language."""
        # Use base extractor for languages without specific extractors
        extractor = MetadataExtractorFactory.create_extractor(language)
        if extractor is None:
            extractor = SimpleMetadataExtractor(language)

        parser = get_parser(language)
        tree = parser.parse(code.encode())

        calls = extractor.extract_calls(tree.root_node, code.encode())
        extracted_names = [call["name"] for call in calls]

        # Check that all expected calls are found
        for expected in expected_calls:
            assert (
                expected in extracted_names
            ), f"Expected call '{expected}' not found in {extracted_names}"

        # Verify span information is present
        for call in calls:
            assert "start" in call
            assert "end" in call
            assert "function_start" in call
            assert "function_end" in call
            assert call["start"] <= call["function_start"]
            assert call["function_end"] <= call["end"]

    def test_java_method_calls(self):
        """Test Java method call extraction."""
        code = """
        public class Test {
            public void test() {
                System.out.println("hello");    // Should detect "println"
                String result = "test".length(); // Should detect "length"
                obj.method();                    // Should detect "method"
                Math.max(1, 2);                  // Should detect "max"
            }
        }
        """
        expected_calls = ["println", "length", "method", "max"]
        self._test_language_calls("java", code, expected_calls)

    def test_ruby_method_calls(self):
        """Test Ruby method call extraction."""
        code = """
        def test
            puts "hello"                    # Should detect "puts"
            result = "test".length          # Should detect "length"
            obj.method                      # Should detect "method"
            [1, 2, 3].map { |x| x * 2 }    # Should detect "map"
        end
        """
        expected_calls = ["puts", "length", "method", "map"]
        self._test_language_calls("ruby", code, expected_calls)

    def test_php_method_calls(self):
        """Test PHP method call extraction."""
        code = """
        <?php
        function test() {
            echo "hello";                 // Should detect "echo"
            $result = strlen("test");     // Should detect "strlen"
            $obj->method();               // Should detect "method"
            array_map($fn, $array);       // Should detect "array_map"
        }
        ?>
        """
        expected_calls = ["echo", "strlen", "method", "array_map"]
        self._test_language_calls("php", code, expected_calls)

    def test_kotlin_method_calls(self):
        """Test Kotlin method call extraction."""
        code = """
        fun test() {
            println("hello")              // Should detect "println"
            val result = "test".length   // Should detect "length"
            obj.method()                  // Should detect "method"
            listOf(1, 2, 3).map { it * 2 } // Should detect "map"
        }
        """
        expected_calls = ["println", "length", "method", "map"]
        self._test_language_calls("kotlin", code, expected_calls)

    def test_swift_method_calls(self):
        """Test Swift method call extraction."""
        code = """
        func test() {
            print("hello")                // Should detect "print"
            let result = "test".count     // Should detect "count"
            obj.method()                  // Should detect "method"
            [1, 2, 3].map { $0 * 2 }     // Should detect "map"
        }
        """
        expected_calls = ["print", "count", "method", "map"]
        self._test_language_calls("swift", code, expected_calls)

    def test_csharp_method_calls(self):
        """Test C# method call extraction."""
        code = """
        public class Test {
            public void test() {
                Console.WriteLine("hello");     // Should detect "WriteLine"
                string result = "test".Length;  // Should detect "Length"
                obj.Method();                    // Should detect "Method"
                Math.Max(1, 2);                  // Should detect "Max"
            }
        }
        """
        expected_calls = ["WriteLine", "Length", "Method", "Max"]
        self._test_language_calls("csharp", code, expected_calls)

    def test_dart_method_calls(self):
        """Test Dart method call extraction."""
        code = """
        void test() {
            print("hello");                 // Should detect "print"
            var result = "test".length;     // Should detect "length"
            obj.method();                   // Should detect "method"
            [1, 2, 3].map((x) => x * 2);   // Should detect "map"
        }
        """
        expected_calls = ["print", "length", "method", "map"]
        self._test_language_calls("dart", code, expected_calls)

    def test_haskell_function_calls(self):
        """Test Haskell function application extraction."""
        code = """
        test :: IO ()
        test = do
            putStrLn "hello"               -- Should detect "putStrLn"
            let result = length "test"      -- Should detect "length"
            let mapped = map id [1,2,3]    -- Should detect "map"
            return ()
        """
        expected_calls = ["putStrLn", "length", "map"]
        self._test_language_calls("haskell", code, expected_calls)

    def test_ocaml_method_calls(self):
        """Test OCaml method call extraction."""
        code = """
        let test () =
            print_endline "hello";         (* Should detect "print_endline" *)
            let result = String.length "test" in (* Should detect "length" *)
            List.map (fun x -> x * 2) [1; 2; 3] (* Should detect "map" *)
        """
        expected_calls = ["print_endline", "length", "map"]
        self._test_language_calls("ocaml", code, expected_calls)

    def test_scala_method_calls(self):
        """Test Scala method call extraction."""
        code = """
        def test(): Unit = {
            println("hello")                // Should detect "println"
            val result = "test".length      // Should detect "length"
            obj.method()                    // Should detect "method"
            List(1, 2, 3).map(_ * 2)       // Should detect "map"
        }
        """
        expected_calls = ["println", "length", "method", "map"]
        self._test_language_calls("scala", code, expected_calls)

    def test_elixir_function_calls(self):
        """Test Elixir function call extraction."""
        code = """
        def test do
            IO.puts("hello")                # Should detect "puts"
            result = String.length("test")  # Should detect "length"
            Enum.map([1, 2, 3], &(&1 * 2)) # Should detect "map"
        end
        """
        expected_calls = ["puts", "length", "map"]
        self._test_language_calls("elixir", code, expected_calls)

    def test_clojure_function_calls(self):
        """Test Clojure function call extraction."""
        code = """
        (defn test []
            (println "hello")               ; Should detect "println"
            (let [result (count "test")]    ; Should detect "count"
                (map #(* % 2) [1 2 3])))    ; Should detect "map"
        """
        expected_calls = ["println", "count", "map"]
        self._test_language_calls("clojure", code, expected_calls)

    def test_julia_function_calls(self):
        """Test Julia function call extraction."""
        code = """
        function test()
            println("hello")                # Should detect "println"
            result = length("test")         # Should detect "length"
            map(x -> x * 2, [1, 2, 3])     # Should detect "map"
        end
        """
        expected_calls = ["println", "length", "map"]
        self._test_language_calls("julia", code, expected_calls)

    def test_r_function_calls(self):
        """Test R function call extraction."""
        code = """
        test <- function() {
            print("hello")                  # Should detect "print"
            result <- length("test")        # Should detect "length"
            sapply(c(1, 2, 3), function(x) x * 2) # Should detect "sapply"
        }
        """
        expected_calls = ["print", "length", "sapply"]
        self._test_language_calls("r", code, expected_calls)

    def test_matlab_function_calls(self):
        """Test MATLAB function call extraction."""
        code = """
        function test()
            disp('hello');                  % Should detect "disp"
            result = length('test');        % Should detect "length"
            arrayfun(@(x) x * 2, [1, 2, 3]); % Should detect "arrayfun"
        end
        """
        expected_calls = ["disp", "length", "arrayfun"]
        self._test_language_calls("matlab", code, expected_calls)


class TestPhase15EdgeCases:
    """Test edge cases for extended language support."""

    @staticmethod
    def _test_no_false_positives(language: str, code: str, not_expected: list[str]):
        """Helper to test that certain patterns are NOT detected as calls."""
        extractor = MetadataExtractorFactory.create_extractor(language)
        if extractor is None:
            extractor = SimpleMetadataExtractor(language)

        parser = get_parser(language)
        tree = parser.parse(code.encode())

        calls = extractor.extract_calls(tree.root_node, code.encode())
        extracted_names = [call["name"] for call in calls]

        # Check that none of the not_expected items are detected
        for item in not_expected:
            assert (
                item not in extracted_names
            ), f"False positive: '{item}' should not be detected as a call"

    def test_java_property_access(self):
        """Test that Java property access is not detected as method calls."""
        code = """
        public class Test {
            public void test() {
                int x = obj.field;          // Should NOT detect "field"
                obj.publicField = 5;        // Should NOT detect "publicField"
            }
        }
        """
        not_expected = ["field", "publicField"]
        self._test_no_false_positives("java", code, not_expected)

    def test_kotlin_property_access(self):
        """Test that Kotlin property access is not detected as method calls."""
        code = """
        fun test() {
            val x = obj.property         // Should NOT detect "property"
            obj.field = 5                // Should NOT detect "field"
        }
        """
        not_expected = ["property", "field"]
        self._test_no_false_positives("kotlin", code, not_expected)

    def test_csharp_property_access(self):
        """Test that C# property access is not detected as method calls."""
        code = """
        public class Test {
            public void test() {
                var x = obj.Property;    // Should NOT detect "Property"
                obj.Field = 5;           // Should NOT detect "Field"
            }
        }
        """
        not_expected = ["Property", "Field"]
        self._test_no_false_positives("csharp", code, not_expected)


class TestPhase15Performance:
    """Test performance of the enhanced base implementation."""

    def test_large_file_performance(self):
        """Test that the enhanced implementation handles large files efficiently."""
        # Generate a large code file with many function calls
        code_lines = []
        for i in range(1000):
            code_lines.append(f"    func{i}();")
            code_lines.append(f"    obj.method{i}();")
            code_lines.append(f"    Module.function{i}();")

        code = "def test():\n" + "\n".join(code_lines)

        extractor = SimpleMetadataExtractor("python")
        parser = get_parser("python")
        tree = parser.parse(code.encode())

        import time

        start = time.time()
        calls = extractor.extract_calls(tree.root_node, code.encode())
        elapsed = time.time() - start

        # Should process 3000 calls in under 1 second
        assert len(calls) >= 3000
        assert (
            elapsed < 1.0
        ), f"Performance issue: took {elapsed:.2f}s to process 3000 calls"

    def test_nested_calls_performance(self):
        """Test performance with deeply nested function calls."""
        # Generate deeply nested calls
        nested = "func("
        for i in range(50):
            nested += f"inner{i}("
        nested += "value"
        nested += ")" * 51

        code = f"def test():\n    {nested}"

        extractor = SimpleMetadataExtractor("python")
        parser = get_parser("python")
        tree = parser.parse(code.encode())

        import time

        start = time.time()
        calls = extractor.extract_calls(tree.root_node, code.encode())
        elapsed = time.time() - start

        # Should handle nested calls efficiently
        assert len(calls) >= 50
        assert (
            elapsed < 0.5
        ), f"Performance issue with nested calls: took {elapsed:.2f}s"



"""Integration tests for the parser module across all languages."""

import threading
import time
from concurrent.futures import ThreadPoolExecutor, as_completed
from pathlib import Path

import pytest

from chunker import (
    ParserConfig,
    clear_cache,
    get_language_info,
    get_parser,
    list_languages,
    return_parser,
)
from chunker._internal.factory import ParserFactory
from chunker._internal.registry import LanguageRegistry
from chunker.exceptions import ParserConfigError

# Note: _factory is no longer directly exposed, tests need updating
# from chunker.parser import _factory
_factory = None  # Tests that rely on this need refactoring


class TestAllLanguages:
    """Test parsing functionality for all supported languages."""

    @staticmethod
    @pytest.fixture(autouse=True)
    def setup_teardown():
        """Setup and teardown for each test."""
        clear_cache()
        yield
        clear_cache()

    def test_all_languages_parse(self):
        """Test that all languages can parse basic code."""
        test_samples = {
            "python": """
def factorial(n: int) -> int:
    ""\"Calculate factorial of n.""\"
    if n <= 1:
        return 1
    return n * factorial(n - 1)

class Calculator:
    def add(self, a: int, b: int) -> int:
        return a + b
""",
            "javascript": """
function factorial(n) {
    if (n <= 1) return 1;
    return n * factorial(n - 1);
}

class Calculator {
    add(a, b) {
        return a + b;
    }
}

const arrow = (x) => x * 2;
""",
            "c": """
#include <stdio.h>

int factorial(int n) {
    if (n <= 1) return 1;
    return n * factorial(n - 1);
}

struct Point {
    int x;
    int y;
};

typedef struct Point Point;
""",
            "cpp": """
#include <iostream>
#include <string>

class Calculator {
public:
    int add(int a, int b) {
        return a + b;
    }

    template<typename T>
    T multiply(T a, T b) {
        return a * b;
    }
};

namespace math {
    int factorial(int n) {
        return (n <= 1) ? 1 : n * factorial(n - 1);
    }
}
""",
            "rust": """
fn factorial(n: u32) -> u32 {
    match n {
        0 | 1 => 1,
        _ => n * factorial(n - 1),
    }
}

struct Calculator {
    value: i32,
}

impl Calculator {
    fn new(value: i32) -> Self {
        Calculator { value }
    }

    fn add(&self, other: i32) -> i32 {
        self.value + other
    }
}

trait Compute {
    fn compute(&self) -> i32;
}
""",
        }
        results = {}
        for lang, code in test_samples.items():
            try:
                parser = get_parser(lang)
                tree = parser.parse(bytes(code, "utf8"))
                root = tree.root_node
                assert root is not None
                assert root.child_count > 0
                has_error = self._has_error_node(root)
                results[lang] = {
                    "success": True,
                    "node_count": self._count_nodes(root),
                    "has_error": has_error,
                    "root_type": root.type,
                }
            except (IndexError, KeyError, SyntaxError) as e:
                results[lang] = {"success": False, "error": str(e)}
        for lang, result in results.items():
            assert result[
                "success"
            ], f"{lang} failed: {result.get('error', 'Unknown error')}"
            assert not result["has_error"], f"{lang} has parsing errors"
            assert result["node_count"] > 10, f"{lang} parsed too few nodes"

    def _has_error_node(self, node):
        """Check if tree contains any ERROR nodes."""
        if node.type == "ERROR" or node.is_error:
            return True
        return any(self._has_error_node(child) for child in node.children)

    def _count_nodes(self, node):
        """Count total nodes in tree."""
        count = 1
        for child in node.children:
            count += self._count_nodes(child)
        return count

    @staticmethod
    def test_language_metadata_consistency():
        """Test that language metadata is consistent across all languages."""
        languages = list_languages()
        assert len(languages) >= 5
        for lang in languages:
            info = get_language_info(lang)
            assert info.name == lang
            assert info.symbol_name == f"tree_sitter_{lang}"
            assert isinstance(info.version, str)
            assert isinstance(info.has_scanner, bool)
            assert isinstance(info.capabilities, dict)
            assert "compatible" in info.capabilities
            assert "language_version" in info.capabilities
            assert info.capabilities["compatible"] is True
            parser = get_parser(lang)
            assert parser is not None


class TestConcurrentParsing:
    """Test concurrent parsing across multiple languages."""

    @classmethod
    def test_concurrent_multi_language_parsing(cls):
        """Test parsing multiple languages concurrently."""
        samples = {
            "python": "def test(): return 42",
            "javascript": "function test() { return 42; }",
            "c": "int test() { return 42; }",
            "cpp": "int test() { return 42; }",
            "rust": "fn test() -> i32 { 42 }",
        }

        def parse_code(lang, code):
            """Parse code and return results."""
            try:
                parser = get_parser(lang)
                tree = parser.parse(bytes(code, "utf8"))
                return_parser(lang, parser)
                return lang, True, tree.root_node.type
            except (OSError, SyntaxError) as e:
                return lang, False, str(e)

        with ThreadPoolExecutor(max_workers=5) as executor:
            futures = {
                executor.submit(parse_code, lang, code): lang
                for lang, code in samples.items()
            }
            results = {}
            for future in as_completed(futures):
                lang, success, data = future.result()
                results[lang] = success, data
        for lang, (success, data) in results.items():
            assert success, f"{lang} failed: {data}"

    @staticmethod
    def test_stress_concurrent_parsing():
        """Stress test with many concurrent parser requests."""
        languages = ["python", "javascript", "rust"]
        iterations = 10
        threads_per_language = 3
        errors = []
        parse_counts = dict.fromkeys(languages, 0)
        lock = threading.Lock()

        def worker(lang, thread_id):
            """Worker thread that repeatedly gets parsers and parses code."""
            code = f"// Thread {thread_id}\nfunction test{thread_id}() {{ return {thread_id}; }}"
            try:
                for _i in range(iterations):
                    parser = get_parser(lang)
                    tree = parser.parse(bytes(code, "utf8"))
                    assert tree.root_node is not None
                    return_parser(lang, parser)
                    with lock:
                        parse_counts[lang] += 1
                    time.sleep(0.001)
            except (OSError, IndexError, KeyError) as e:
                errors.append((lang, thread_id, str(e)))

        threads = []
        for lang in languages:
            for i in range(threads_per_language):
                t = threading.Thread(target=worker, args=(lang, i))
                threads.append(t)
                t.start()
        for t in threads:
            t.join()
        assert len(errors) == 0, f"Errors occurred: {errors}"
        for lang, count in parse_counts.items():
            expected = iterations * threads_per_language
            assert count == expected, f"{lang}: expected {expected} parses, got {count}"


class TestParserConfiguration:
    """Test parser configuration across languages."""

    @classmethod
    def test_timeout_configuration(cls):
        """Test timeout configuration for all languages."""
        languages = list_languages()
        config = ParserConfig(timeout_ms=100)
        for lang in languages:
            parser = get_parser(lang, config)
            tree = parser.parse(b"test")
            assert tree is not None

    @classmethod
    def test_invalid_configurations(cls):
        """Test that invalid configurations are rejected."""
        invalid_configs = [
            ParserConfig(timeout_ms=-1),
            ParserConfig(timeout_ms="not a number"),
            ParserConfig(included_ranges="not a list"),
        ]
        for config in invalid_configs:
            with pytest.raises(ParserConfigError):
                get_parser("python", config)


class TestMemoryEfficiency:
    """Test memory efficiency of parser caching and pooling."""

    @classmethod
    def test_parser_reuse(cls):
        """Test that parsers are properly reused from cache/pool."""
        if _factory is None:
            lib_path = (
                Path(
                    __file__,
                ).parent.parent
                / "build"
                / "my-languages.so"
            )
            test_registry = LanguageRegistry(lib_path)
            test_factory = ParserFactory(test_registry)
            initial_stats = test_factory.get_stats()
            initial_count = initial_stats["total_parsers_created"]
            for _ in range(10):
                parser = test_factory.get_parser("python")
                test_factory.return_parser("python", parser)
            [test_factory.get_parser("python") for _ in range(3)]
            final_stats = test_factory.get_stats()
            final_count = final_stats["total_parsers_created"]
        else:
            initial_stats = _factory.get_stats()
            initial_count = initial_stats["total_parsers_created"]
            for _ in range(10):
                parser = get_parser("python")
                return_parser("python", parser)
            [get_parser("python") for _ in range(3)]
            final_stats = _factory.get_stats()
            final_count = final_stats["total_parsers_created"]
        parsers_created = final_count - initial_count
        assert parsers_created <= 5, f"Created too many parsers: {parsers_created}"

    @classmethod
    def test_cache_effectiveness(cls):
        """Test cache hit rate under typical usage."""
        lib_path = Path(__file__).parent.parent / "build" / "my-languages.so"
        test_registry = LanguageRegistry(lib_path)
        test_factory = ParserFactory(test_registry, cache_size=10)
        languages = ["python", "javascript", "rust"]
        initial_stats = test_factory.get_stats()
        initial_count = initial_stats["total_parsers_created"]
        for _ in range(5):
            for lang in languages:
                parser = test_factory.get_parser(lang)
                tree = parser.parse(b"test")
                assert tree is not None
        final_stats = test_factory.get_stats()
        parsers_created = final_stats["total_parsers_created"] - initial_count
        assert parsers_created == len(
            languages,
        ), f"Cache not effective: created {parsers_created} parsers for {len(languages)} languages"


class TestErrorScenarios:
    """Test error handling in integration scenarios."""

    @staticmethod
    def test_large_file_parsing():
        """Test parsing large files."""
        code_parts = ['"""Large test file."""\n']
        code_parts.extend(
            f"""
def function_{i}(param_{i}: int) -> int:
    ""\"Function {i} docstring.""\"
    result = param_{i} * 2
    if result > 100:
        return result - 10
    return result + 10

class Class_{i}:
    ""\"Class {i} docstring.""\"

    def method_{i}(self, value: int) -> str:
        ""\"Method {i} docstring.""\"
        return f"Value: {{value}}\"
"""
            for i in range(1000)
        )
        large_code = "\n".join(code_parts)
        parser = get_parser("python")
        start_time = time.time()
        tree = parser.parse(bytes(large_code, "utf8"))
        parse_time = time.time() - start_time
        assert tree is not None
        assert tree.root_node.child_count > 0
        assert parse_time < 1.0, f"Parsing took too long: {parse_time:.2f}s"

    @staticmethod
    def test_malformed_code_handling():
        """Test handling of malformed code."""
        malformed_samples = {
            "python": 'def incomplete_function(\n    print("unclosed',
            "javascript": "function test() { return }",
            "c": "int main() { return }",
            "cpp": "class Test { public: int get( };",
            "rust": 'fn test() -> { println!("incomplete',
        }
        for lang, code in malformed_samples.items():
            parser = get_parser(lang)
            tree = parser.parse(bytes(code, "utf8"))
            assert tree is not None
            assert tree.root_node is not None


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""
Tree-sitter Chunker - Semantic code chunking for LLMs and embeddings.

Simple usage:
    from chunker import chunk_file, chunk_text, chunk_directory

    # Chunk a single file
    chunks = chunk_file("example.py", language="python")

    # Chunk raw text
    chunks = chunk_text(code_text, language="javascript")

    # Chunk entire directory
    results = chunk_directory("src/", language="python")
"""

from ._internal.cache import ASTCache
from .chunker import chunk_text_with_token_limit, count_chunk_tokens
from .chunker_config import ChunkerConfig
from .core import chunk_file
from .exceptions import LanguageNotFoundError, LibraryNotFoundError, ParserError
from .fallback.intelligent_fallback import (
    ChunkingDecision,
    DecisionMetrics,
    IntelligentFallbackChunker,
)
from .incremental import (
    DefaultChangeDetector,
    DefaultChunkCache,
    DefaultIncrementalProcessor,
)
from .languages.base import PluginConfig
from .multi_language import (
    LanguageDetectorImpl,
    MultiLanguageProcessorImpl,
    ProjectAnalyzerImpl,
)
from .optimization import (
    ChunkBoundaryAnalyzer,
    ChunkOptimizer,
    OptimizationConfig,
    OptimizationMetrics,
    OptimizationStrategy,
)
from .parallel import chunk_directory_parallel as chunk_directory
from .parser import (
    ParserConfig,
    clear_cache,
    get_language_info,
    get_parser,
    list_languages,
    return_parser,
)
from .plugin_manager import PluginManager, get_plugin_manager
from .semantic_query import SemanticQuery

# Optional advanced query features (may require heavy deps like numpy)
try:
    from .query_advanced import (
        AdvancedQueryIndex,
        NaturalLanguageQueryEngine,
        SmartQueryOptimizer,
    )

    _ADV_QUERY_AVAILABLE = True
except Exception:  # pragma: no cover - keep import lightweight during build tests
    AdvancedQueryIndex = None  # type: ignore[assignment]
    NaturalLanguageQueryEngine = None  # type: ignore[assignment]
    SmartQueryOptimizer = None  # type: ignore[assignment]
    _ADV_QUERY_AVAILABLE = False

# Optional system integration (Phase 1.7 + 1.8 integration)
try:
    from .integration import (
        SystemIntegrator,
        get_system_health,
        get_system_integrator,
        initialize_treesitter_system,
        process_grammar_error,
    )

    _SYSTEM_INTEGRATION_AVAILABLE = True
except (
    Exception
):  # pragma: no cover - graceful degradation when integration not available
    SystemIntegrator = None  # type: ignore[assignment]
    get_system_integrator = None  # type: ignore[assignment]
    initialize_treesitter_system = None  # type: ignore[assignment]
    process_grammar_error = None  # type: ignore[assignment]
    get_system_health = None  # type: ignore[assignment]
    _SYSTEM_INTEGRATION_AVAILABLE = False
from .smart_context import InMemoryContextCache, TreeSitterSmartContextProvider
from .streaming import chunk_file_streaming
from .types import CodeChunk

# Dynamic version from package metadata
try:
    from importlib.metadata import version

    __version__ = version("treesitter-chunker")
except ImportError:
    __version__ = "1.0.8"  # Fallback version


# Simple text chunking
def chunk_text(text: str, language: str, **kwargs):
    """Chunk text content directly without file I/O."""
    import tempfile
    from pathlib import Path

    # Write to temporary file and chunk it
    with tempfile.NamedTemporaryFile(
        encoding="utf-8",
        mode="w",
        suffix=".tmp",
        delete=False,
    ) as f:
        f.write(text)
        temp_path = f.name

    try:
        chunks = chunk_file(temp_path, language, **kwargs)
        return chunks
    finally:
        Path(temp_path).unlink(missing_ok=True)


# Convenient exports for common use cases
__all__ = [
    "ASTCache",
    "AdvancedQueryIndex",
    "ChunkBoundaryAnalyzer",
    "ChunkOptimizer",
    "ChunkerConfig",
    "ChunkingDecision",
    "CodeChunk",
    "DecisionMetrics",
    "DefaultChangeDetector",
    "DefaultChunkCache",
    "DefaultIncrementalProcessor",
    "InMemoryContextCache",
    "IntelligentFallbackChunker",
    "LanguageDetectorImpl",
    "LanguageNotFoundError",
    "LibraryNotFoundError",
    "MultiLanguageProcessorImpl",
]

# Extend __all__ with advanced query API only if import succeeded
if _ADV_QUERY_AVAILABLE:
    __all__.extend(
        [
            "AdvancedQueryIndex",
            "NaturalLanguageQueryEngine",
            "SmartQueryOptimizer",
        ],
    )

# Extend __all__ with system integration API only if import succeeded
if _SYSTEM_INTEGRATION_AVAILABLE:
    __all__.extend(
        [
            "SystemIntegrator",
            "get_system_health",
            "get_system_integrator",
            "initialize_treesitter_system",
            "process_grammar_error",
        ],
    )

__all__.extend(
    [
        "OptimizationConfig",
        "OptimizationMetrics",
        "OptimizationStrategy",
        "ParserConfig",
        "ParserError",
        "PluginConfig",
        "PluginManager",
        "ProjectAnalyzerImpl",
        "SemanticQuery",
        "TreeSitterSmartContextProvider",
        "__version__",
        "chunk_directory",
        "chunk_file",
        "chunk_file_streaming",
        "chunk_text",
        "chunk_text_with_token_limit",
        "clear_cache",
        "count_chunk_tokens",
        "get_language_info",
        "get_parser",
        "get_plugin_manager",
        "list_languages",
        "return_parser",
    ],
)



"""Comprehensive tests for chunk hierarchy building and navigation."""

import pytest

from chunker.core import chunk_text
from chunker.hierarchy.builder import ChunkHierarchyBuilder
from chunker.hierarchy.navigator import HierarchyNavigator
from chunker.types import CodeChunk


class TestChunkHierarchyBuilder:
    """Test the ChunkHierarchyBuilder class."""

    def setup_method(self):
        """Set up test fixtures."""
        self.builder = ChunkHierarchyBuilder()
        self.chunks = self._create_test_chunks()

    @classmethod
    def _create_test_chunks(cls) -> list[CodeChunk]:
        """Create a test hierarchy of chunks.

        Structure:
        - module (root)
          - class TestClass
            - method test_method1
            - method test_method2
          - function helper_func
        - function standalone_func (root)
        """
        module_chunk = CodeChunk(
            language="python",
            file_path="test.py",
            node_type="module",
            start_line=1,
            end_line=20,
            byte_start=0,
            byte_end=500,
            parent_context="",
            content="# module content",
            chunk_id="module_1",
            parent_chunk_id=None,
        )
        class_chunk = CodeChunk(
            language="python",
            file_path="test.py",
            node_type="class_definition",
            start_line=3,
            end_line=15,
            byte_start=50,
            byte_end=400,
            parent_context="module",
            content="class TestClass:",
            chunk_id="class_1",
            parent_chunk_id="module_1",
        )
        method1_chunk = CodeChunk(
            language="python",
            file_path="test.py",
            node_type="function_definition",
            start_line=5,
            end_line=8,
            byte_start=100,
            byte_end=200,
            parent_context="class_definition",
            content="def test_method1(self):",
            chunk_id="method_1",
            parent_chunk_id="class_1",
        )
        method2_chunk = CodeChunk(
            language="python",
            file_path="test.py",
            node_type="function_definition",
            start_line=10,
            end_line=13,
            byte_start=250,
            byte_end=350,
            parent_context="class_definition",
            content="def test_method2(self):",
            chunk_id="method_2",
            parent_chunk_id="class_1",
        )
        helper_chunk = CodeChunk(
            language="python",
            file_path="test.py",
            node_type="function_definition",
            start_line=17,
            end_line=19,
            byte_start=420,
            byte_end=480,
            parent_context="module",
            content="def helper_func():",
            chunk_id="helper_1",
            parent_chunk_id="module_1",
        )
        standalone_chunk = CodeChunk(
            language="python",
            file_path="test.py",
            node_type="function_definition",
            start_line=22,
            end_line=25,
            byte_start=520,
            byte_end=600,
            parent_context="",
            content="def standalone_func():",
            chunk_id="standalone_1",
            parent_chunk_id=None,
        )
        return [
            module_chunk,
            class_chunk,
            method1_chunk,
            method2_chunk,
            helper_chunk,
            standalone_chunk,
        ]

    def test_build_hierarchy_basic(self):
        """Test building a basic hierarchy."""
        hierarchy = self.builder.build_hierarchy(self.chunks)
        assert len(hierarchy.root_chunks) == 2
        assert "module_1" in hierarchy.root_chunks
        assert "standalone_1" in hierarchy.root_chunks
        assert hierarchy.parent_map["class_1"] == "module_1"
        assert hierarchy.parent_map["method_1"] == "class_1"
        assert hierarchy.parent_map["method_2"] == "class_1"
        assert hierarchy.parent_map["helper_1"] == "module_1"
        assert "module_1" not in hierarchy.parent_map
        assert "standalone_1" not in hierarchy.parent_map
        assert set(hierarchy.children_map["module_1"]) == {"class_1", "helper_1"}
        assert set(hierarchy.children_map["class_1"]) == {"method_1", "method_2"}
        assert "method_1" not in hierarchy.children_map
        assert "standalone_1" not in hierarchy.children_map

    def test_build_hierarchy_empty(self):
        """Test building hierarchy with empty chunk list."""
        hierarchy = self.builder.build_hierarchy([])
        assert hierarchy.root_chunks == []
        assert hierarchy.parent_map == {}
        assert hierarchy.children_map == {}
        assert hierarchy.chunk_map == {}

    def test_build_hierarchy_orphaned_chunks(self):
        """Test handling chunks with missing parents."""
        chunks = [
            CodeChunk(
                language="python",
                file_path="test.py",
                node_type="function_definition",
                start_line=1,
                end_line=3,
                byte_start=0,
                byte_end=50,
                parent_context="",
                content="def func():",
                chunk_id="func_1",
                parent_chunk_id="missing_parent",
            ),
        ]
        hierarchy = self.builder.build_hierarchy(chunks)
        assert hierarchy.root_chunks == ["func_1"]
        assert hierarchy.parent_map == {"func_1": "missing_parent"}

    def test_find_common_ancestor_same_chunk(self):
        """Test finding common ancestor when both chunks are the same."""
        hierarchy = self.builder.build_hierarchy(self.chunks)
        method1 = self.chunks[2]
        ancestor = self.builder.find_common_ancestor(method1, method1, hierarchy)
        assert ancestor == "method_1"

    def test_find_common_ancestor_siblings(self):
        """Test finding common ancestor of sibling chunks."""
        hierarchy = self.builder.build_hierarchy(self.chunks)
        method1 = self.chunks[2]
        method2 = self.chunks[3]
        ancestor = self.builder.find_common_ancestor(method1, method2, hierarchy)
        assert ancestor == "class_1"

    def test_find_common_ancestor_parent_child(self):
        """Test finding common ancestor of parent-child chunks."""
        hierarchy = self.builder.build_hierarchy(self.chunks)
        class_chunk = self.chunks[1]
        method1 = self.chunks[2]
        ancestor = self.builder.find_common_ancestor(class_chunk, method1, hierarchy)
        assert ancestor == "class_1"

    def test_find_common_ancestor_no_common(self):
        """Test finding common ancestor of chunks in different trees."""
        hierarchy = self.builder.build_hierarchy(self.chunks)
        method1 = self.chunks[2]
        standalone = self.chunks[5]
        ancestor = self.builder.find_common_ancestor(method1, standalone, hierarchy)
        assert ancestor is None

    def test_validate_hierarchy_valid(self):
        """Test validation of a valid hierarchy."""
        hierarchy = self.builder.build_hierarchy(self.chunks)
        assert self.builder.validate_hierarchy(hierarchy) is True

    def test_validate_hierarchy_cycle(self):
        """Test detection of cycles in hierarchy."""
        hierarchy = self.builder.build_hierarchy(self.chunks)
        hierarchy.parent_map["module_1"] = "class_1"
        with pytest.raises(ValueError, match="Cycle detected"):
            self.builder.validate_hierarchy(hierarchy)

    def test_validate_hierarchy_inconsistent_maps(self):
        """Test detection of inconsistent parent/children maps."""
        hierarchy = self.builder.build_hierarchy(self.chunks)
        hierarchy.children_map["class_1"].remove("method_1")
        with pytest.raises(ValueError, match="not in parent"):
            self.builder.validate_hierarchy(hierarchy)

    def test_chunk_ordering(self):
        """Test that chunks are ordered by start line."""
        chunks = list(reversed(self.chunks))
        hierarchy = self.builder.build_hierarchy(chunks)
        assert hierarchy.root_chunks[0] == "module_1"
        assert hierarchy.root_chunks[1] == "standalone_1"
        assert hierarchy.children_map["class_1"][0] == "method_1"
        assert hierarchy.children_map["class_1"][1] == "method_2"


class TestHierarchyNavigator:
    """Test the HierarchyNavigator class."""

    def setup_method(self):
        """Set up test fixtures."""
        self.navigator = HierarchyNavigator()
        self.builder = ChunkHierarchyBuilder()
        self.chunks = self._create_test_chunks()
        self.hierarchy = self.builder.build_hierarchy(self.chunks)

    @classmethod
    def _create_test_chunks(cls) -> list[CodeChunk]:
        """Create the same test hierarchy as builder tests."""
        return TestChunkHierarchyBuilder()._create_test_chunks()

    def test_get_children(self):
        """Test getting direct children of a chunk."""
        children = self.navigator.get_children("class_1", self.hierarchy)
        assert len(children) == 2
        assert children[0].chunk_id == "method_1"
        assert children[1].chunk_id == "method_2"
        children = self.navigator.get_children("method_1", self.hierarchy)
        assert children == []
        children = self.navigator.get_children("nonexistent", self.hierarchy)
        assert children == []

    def test_get_descendants(self):
        """Test getting all descendants of a chunk."""
        descendants = self.navigator.get_descendants(
            "module_1",
            self.hierarchy,
        )
        assert len(descendants) == 4
        descendant_ids = {d.chunk_id for d in descendants}
        assert descendant_ids == {
            "class_1",
            "method_1",
            "method_2",
            "helper_1",
        }
        descendants = self.navigator.get_descendants("class_1", self.hierarchy)
        assert len(descendants) == 2
        assert descendants[0].chunk_id == "method_1"
        assert descendants[1].chunk_id == "method_2"
        descendants = self.navigator.get_descendants(
            "method_1",
            self.hierarchy,
        )
        assert descendants == []

    def test_get_ancestors(self):
        """Test getting all ancestors of a chunk."""
        ancestors = self.navigator.get_ancestors("method_1", self.hierarchy)
        assert len(ancestors) == 2
        assert ancestors[0].chunk_id == "class_1"
        assert ancestors[1].chunk_id == "module_1"
        ancestors = self.navigator.get_ancestors("module_1", self.hierarchy)
        assert ancestors == []

    def test_get_siblings(self):
        """Test getting sibling chunks."""
        siblings = self.navigator.get_siblings("method_1", self.hierarchy)
        assert len(siblings) == 1
        assert siblings[0].chunk_id == "method_2"
        siblings = self.navigator.get_siblings("class_1", self.hierarchy)
        assert len(siblings) == 1
        assert siblings[0].chunk_id == "helper_1"
        siblings = self.navigator.get_siblings("module_1", self.hierarchy)
        assert len(siblings) == 1
        assert siblings[0].chunk_id == "standalone_1"

    def test_filter_by_depth(self):
        """Test filtering chunks by depth."""
        chunks = self.navigator.filter_by_depth(
            self.hierarchy,
            min_depth=0,
            max_depth=0,
        )
        assert len(chunks) == 2
        chunk_ids = {c.chunk_id for c in chunks}
        assert chunk_ids == {"module_1", "standalone_1"}
        chunks = self.navigator.filter_by_depth(
            self.hierarchy,
            min_depth=1,
            max_depth=1,
        )
        assert len(chunks) == 2
        chunk_ids = {c.chunk_id for c in chunks}
        assert chunk_ids == {"class_1", "helper_1"}
        chunks = self.navigator.filter_by_depth(
            self.hierarchy,
            min_depth=2,
            max_depth=2,
        )
        assert len(chunks) == 2
        chunk_ids = {c.chunk_id for c in chunks}
        assert chunk_ids == {"method_1", "method_2"}
        chunks = self.navigator.filter_by_depth(self.hierarchy, min_depth=1)
        assert len(chunks) == 4
        chunks = self.navigator.filter_by_depth(self.hierarchy)
        assert len(chunks) == 6

    def test_get_depth(self):
        """Test getting depth of chunks."""
        assert self.hierarchy.get_depth("module_1") == 0
        assert self.hierarchy.get_depth("standalone_1") == 0
        assert self.hierarchy.get_depth("class_1") == 1
        assert self.hierarchy.get_depth("helper_1") == 1
        assert self.hierarchy.get_depth("method_1") == 2
        assert self.hierarchy.get_depth("method_2") == 2

    def test_get_subtree(self):
        """Test extracting a subtree."""
        subtree = self.navigator.get_subtree("class_1", self.hierarchy)
        assert subtree.root_chunks == ["class_1"]
        assert len(subtree.chunk_map) == 3
        assert set(subtree.chunk_map.keys()) == {"class_1", "method_1", "method_2"}
        assert subtree.children_map["class_1"] == ["method_1", "method_2"]
        assert "method_1" in subtree.parent_map
        assert subtree.parent_map["method_1"] == "class_1"
        subtree = self.navigator.get_subtree("method_1", self.hierarchy)
        assert subtree.root_chunks == ["method_1"]
        assert len(subtree.chunk_map) == 1
        assert subtree.parent_map == {}
        assert subtree.children_map == {}
        subtree = self.navigator.get_subtree("nonexistent", self.hierarchy)
        assert subtree.root_chunks == []
        assert subtree.chunk_map == {}

    def test_get_level_order_traversal(self):
        """Test level-order traversal of hierarchy."""
        levels = self.navigator.get_level_order_traversal(self.hierarchy)
        assert len(levels) == 3
        assert len(levels[0]) == 2
        level0_ids = {c.chunk_id for c in levels[0]}
        assert level0_ids == {"module_1", "standalone_1"}
        assert len(levels[1]) == 2
        level1_ids = {c.chunk_id for c in levels[1]}
        assert level1_ids == {"class_1", "helper_1"}
        assert len(levels[2]) == 2
        level2_ids = {c.chunk_id for c in levels[2]}
        assert level2_ids == {"method_1", "method_2"}

    def test_find_chunks_by_type(self):
        """Test finding chunks by node type."""
        functions = self.navigator.find_chunks_by_type(
            "function_definition",
            self.hierarchy,
        )
        assert len(functions) == 4
        func_ids = {f.chunk_id for f in functions}
        assert func_ids == {"method_1", "method_2", "helper_1", "standalone_1"}
        classes = self.navigator.find_chunks_by_type("class_definition", self.hierarchy)
        assert len(classes) == 1
        assert classes[0].chunk_id == "class_1"
        functions = self.navigator.find_chunks_by_type(
            "function_definition",
            self.hierarchy,
            subtree_root="class_1",
        )
        assert len(functions) == 2
        func_ids = {f.chunk_id for f in functions}
        assert func_ids == {"method_1", "method_2"}
        results = self.navigator.find_chunks_by_type("nonexistent_type", self.hierarchy)
        assert results == []


class TestHierarchyIntegration:
    """Integration tests using real Tree-sitter parsing."""

    def setup_method(self):
        """Set up test fixtures."""
        self.builder = ChunkHierarchyBuilder()
        self.navigator = HierarchyNavigator()

    def test_python_code_hierarchy(self):
        """Test hierarchy building with real Python code."""
        python_code = """
class Calculator:
    ""\"A simple calculator class.""\"

    def __init__(self):
        self.result = 0

    def add(self, x, y):
        ""\"Add two numbers.""\"
        return x + y

    def multiply(self, x, y):
        ""\"Multiply two numbers.""\"
        return x * y

    class NestedClass:
        def nested_method(self):
            pass

def helper_function():
    ""\"A helper function.""\"
    return 42

async def async_function():
    ""\"An async function.""\"
    pass
"""
        chunks = chunk_text(python_code, "python", "calculator.py")
        hierarchy = self.builder.build_hierarchy(chunks)
        assert len(hierarchy.root_chunks) >= 3
        calc_chunks = self.navigator.find_chunks_by_type("class_definition", hierarchy)
        calc_chunk = next((c for c in calc_chunks if "Calculator" in c.content), None)
        assert calc_chunk is not None
        calc_children = self.navigator.get_children(calc_chunk.chunk_id, hierarchy)
        calc_child_types = {c.node_type for c in calc_children}
        assert "function_definition" in calc_child_types
        depth_0 = self.navigator.filter_by_depth(hierarchy, 0, 0)
        depth_1 = self.navigator.filter_by_depth(hierarchy, 1, 1)
        assert len(depth_0) > 0
        assert len(depth_1) > 0

    def test_javascript_code_hierarchy(self):
        """Test hierarchy building with JavaScript code."""
        js_code = """
class Component {
    constructor() {
        this.state = {};
    }

    render() {
        return null;
    }

    static defaultProps = {
        name: "Component"
    };
}

function processData(data) {
    return data.map(item => {
        return item * 2;
    });
}

const arrowFunc = () => {
    console.log("Hello");
};
"""
        chunks = chunk_text(js_code, "javascript", "component.js")
        hierarchy = self.builder.build_hierarchy(chunks)
        assert len(hierarchy.chunk_map) > 0
        root_funcs = [
            hierarchy.chunk_map[cid]
            for cid in hierarchy.root_chunks
            if cid in hierarchy.chunk_map
            and hierarchy.chunk_map[cid].node_type
            in {"function_declaration", "arrow_function"}
        ]
        assert len(root_funcs) >= 1

    def test_complex_nesting(self):
        """Test deeply nested structures."""
        chunks = []
        for i in range(5):
            chunk = CodeChunk(
                language="python",
                file_path="nested.py",
                node_type="class_definition" if i % 2 == 0 else "function_definition",
                start_line=i * 10 + 1,
                end_line=(i + 1) * 10,
                byte_start=i * 100,
                byte_end=(i + 1) * 100,
                parent_context="",
                content=f"level_{i}",
                chunk_id=f"chunk_{i}",
                parent_chunk_id=f"chunk_{i - 1}" if i > 0 else None,
            )
            chunks.append(chunk)
        hierarchy = self.builder.build_hierarchy(chunks)
        assert hierarchy.get_depth("chunk_0") == 0
        assert hierarchy.get_depth("chunk_4") == 4
        ancestors = self.navigator.get_ancestors("chunk_4", hierarchy)
        assert len(ancestors) == 4
        assert ancestors[0].chunk_id == "chunk_3"
        assert ancestors[-1].chunk_id == "chunk_0"
        subtree = self.navigator.get_subtree("chunk_2", hierarchy)
        assert len(subtree.chunk_map) == 3
        assert subtree.root_chunks == ["chunk_2"]



"""Unit tests for incremental processing without tree-sitter dependency."""

import shutil
import tempfile
from datetime import datetime, timedelta
from unittest.mock import patch

import pytest

from chunker.incremental import (
    DefaultChangeDetector,
    DefaultChunkCache,
    DefaultIncrementalProcessor,
    SimpleIncrementalIndex,
)
from chunker.interfaces.incremental import ChangeType, ChunkChange, ChunkDiff
from chunker.types import CodeChunk


@pytest.fixture
def sample_chunks():
    """Create sample chunks for testing."""
    return [
        CodeChunk(
            language="python",
            file_path="test.py",
            node_type="function_definition",
            start_line=1,
            end_line=5,
            byte_start=0,
            byte_end=100,
            parent_context="",
            content="""def hello():
    print('Hello')
    return True
""",
            chunk_id="chunk1",
        ),
        CodeChunk(
            language="python",
            file_path="test.py",
            node_type="function_definition",
            start_line=7,
            end_line=10,
            byte_start=101,
            byte_end=200,
            parent_context="",
            content="""def world():
    print('World')
""",
            chunk_id="chunk2",
        ),
        CodeChunk(
            language="python",
            file_path="test.py",
            node_type="class_definition",
            start_line=12,
            end_line=20,
            byte_start=201,
            byte_end=400,
            parent_context="",
            content="""class MyClass:
    def __init__(self):
        pass
""",
            chunk_id="chunk3",
        ),
    ]


@pytest.fixture
def temp_cache_dir():
    """Create temporary cache directory."""
    temp_dir = tempfile.mkdtemp()
    yield temp_dir
    shutil.rmtree(temp_dir)


class TestIncrementalProcessor:
    """Test incremental processor implementation."""

    @classmethod
    @patch("chunker.incremental.chunk_text")
    def test_compute_diff_no_changes(cls, mock_chunk_text, sample_chunks):
        """Test diff computation with no changes."""
        mock_chunk_text.return_value = sample_chunks
        processor = DefaultIncrementalProcessor()
        content = "\n".join([chunk.content for chunk in sample_chunks])
        diff = processor.compute_diff(sample_chunks, content, "python")
        assert len(diff.unchanged_chunks) == len(sample_chunks)
        assert len(diff.added_chunks) == 0
        assert len(diff.deleted_chunks) == 0
        assert len(diff.modified_chunks) == 0

    @classmethod
    @patch("chunker.incremental.chunk_text")
    def test_compute_diff_with_modifications(cls, mock_chunk_text, sample_chunks):
        """Test diff computation with modified content."""
        modified_chunk = CodeChunk(
            language="python",
            file_path="test.py",
            node_type="function_definition",
            start_line=1,
            end_line=5,
            byte_start=0,
            byte_end=100,
            parent_context="",
            content="""def hello():
    print('Hello, World!')
    return True
""",
            chunk_id="chunk1",
        )
        mock_chunk_text.return_value = [modified_chunk, *sample_chunks[1:]]
        processor = DefaultIncrementalProcessor()
        modified_content = "modified content"
        diff = processor.compute_diff(
            sample_chunks,
            modified_content,
            "python",
        )
        assert len(diff.modified_chunks) == 1
        assert diff.summary["modified"] == 1

    @classmethod
    @patch("chunker.incremental.chunk_text")
    def test_compute_diff_with_additions(cls, mock_chunk_text, sample_chunks):
        """Test diff computation with added content."""
        new_chunk = CodeChunk(
            language="python",
            file_path="test.py",
            node_type="function_definition",
            start_line=22,
            end_line=25,
            byte_start=401,
            byte_end=500,
            parent_context="",
            content="""def new_function():
    pass
""",
            chunk_id="chunk4",
        )
        mock_chunk_text.return_value = [*sample_chunks, new_chunk]
        processor = DefaultIncrementalProcessor()
        diff = processor.compute_diff(sample_chunks, "modified content", "python")
        assert len(diff.added_chunks) == 1
        assert diff.summary["added"] == 1

    @classmethod
    @patch("chunker.incremental.chunk_text")
    def test_compute_diff_with_deletions(cls, mock_chunk_text, sample_chunks):
        """Test diff computation with deleted content."""
        mock_chunk_text.return_value = [sample_chunks[0], sample_chunks[2]]
        processor = DefaultIncrementalProcessor()
        diff = processor.compute_diff(sample_chunks, "modified content", "python")
        assert len(diff.deleted_chunks) == 1
        assert diff.summary["deleted"] == 1

    @classmethod
    def test_detect_moved_chunks(cls, sample_chunks):
        """Test detection of moved chunks."""
        processor = DefaultIncrementalProcessor()
        old_chunks = [sample_chunks[0]]
        new_chunks = [
            CodeChunk(
                language="python",
                file_path="test.py",
                node_type="function_definition",
                start_line=10,
                end_line=14,
                byte_start=200,
                byte_end=300,
                parent_context="",
                content=sample_chunks[0].content,
                chunk_id="chunk1_moved",
            ),
        ]
        moved_pairs = processor.detect_moved_chunks(old_chunks, new_chunks)
        assert len(moved_pairs) == 1
        assert moved_pairs[0][0].content == moved_pairs[0][1].content
        assert moved_pairs[0][0].start_line != moved_pairs[0][1].start_line

    @classmethod
    def test_update_chunks(cls, sample_chunks):
        """Test updating chunks based on diff."""
        processor = DefaultIncrementalProcessor()
        added_chunk = CodeChunk(
            language="python",
            file_path="test.py",
            node_type="function_definition",
            start_line=25,
            end_line=28,
            byte_start=500,
            byte_end=600,
            parent_context="",
            content="""def added():
    pass
""",
            chunk_id="added_chunk",
        )
        diff = ChunkDiff(
            changes=[
                ChunkChange(
                    chunk_id="chunk2",
                    change_type=ChangeType.DELETED,
                    old_chunk=sample_chunks[1],
                    new_chunk=None,
                    line_changes=[(7, 10)],
                    confidence=1.0,
                ),
                ChunkChange(
                    chunk_id="added_chunk",
                    change_type=ChangeType.ADDED,
                    old_chunk=None,
                    new_chunk=added_chunk,
                    line_changes=[(25, 28)],
                    confidence=1.0,
                ),
            ],
            added_chunks=[added_chunk],
            deleted_chunks=[sample_chunks[1]],
            modified_chunks=[],
            unchanged_chunks=[sample_chunks[0], sample_chunks[2]],
            summary={"added": 1, "deleted": 1, "modified": 0, "unchanged": 2},
        )
        updated_chunks = processor.update_chunks(sample_chunks, diff)
        assert len(updated_chunks) == 3
        chunk_ids = [c.chunk_id for c in updated_chunks]
        assert "chunk1" in chunk_ids
        assert "chunk2" not in chunk_ids
        assert "chunk3" in chunk_ids
        assert "added_chunk" in chunk_ids

    @classmethod
    def test_merge_incremental_results(cls, sample_chunks):
        """Test merging incremental results."""
        processor = DefaultIncrementalProcessor()
        incremental_chunks = [
            CodeChunk(
                language="python",
                file_path="test.py",
                node_type="function_definition",
                start_line=7,
                end_line=10,
                byte_start=101,
                byte_end=200,
                parent_context="",
                content="""def world_modified():
    print('Modified')
""",
                chunk_id="chunk2_new",
            ),
        ]
        changed_regions = [(7, 10)]
        merged = processor.merge_incremental_results(
            sample_chunks,
            incremental_chunks,
            changed_regions,
        )
        assert len(merged) > 0
        old_chunk2_ids = [c.chunk_id for c in merged if c.chunk_id == "chunk2"]
        assert len(old_chunk2_ids) == 0
        new_chunk_ids = [c.chunk_id for c in merged if c.chunk_id == "chunk2_new"]
        assert len(new_chunk_ids) == 1


class TestChunkCache:
    """Test chunk cache implementation."""

    @classmethod
    def test_store_and_retrieve(cls, sample_chunks, temp_cache_dir):
        """Test storing and retrieving chunks."""
        cache = DefaultChunkCache(temp_cache_dir)
        file_hash = "test_hash_123"
        cache.store("test.py", sample_chunks, file_hash)
        entry = cache.retrieve("test.py", file_hash)
        assert entry is not None
        assert entry.file_path == "test.py"
        assert entry.file_hash == file_hash
        assert len(entry.chunks) == len(sample_chunks)

    @classmethod
    def test_retrieve_with_wrong_hash(cls, sample_chunks, temp_cache_dir):
        """Test retrieving with wrong hash."""
        cache = DefaultChunkCache(temp_cache_dir)
        cache.store("test.py", sample_chunks, "correct_hash")
        entry = cache.retrieve("test.py", "wrong_hash")
        assert entry is None

    @classmethod
    def test_invalidate_specific_file(cls, sample_chunks, temp_cache_dir):
        """Test invalidating specific file."""
        cache = DefaultChunkCache(temp_cache_dir)
        cache.store("test1.py", sample_chunks, "hash1")
        cache.store("test2.py", sample_chunks, "hash2")
        count = cache.invalidate("test1.py")
        assert count == 1
        assert cache.retrieve("test1.py") is None
        assert cache.retrieve("test2.py") is not None

    @classmethod
    def test_invalidate_by_age(cls, sample_chunks, temp_cache_dir):
        """Test invalidating by age."""
        cache = DefaultChunkCache(temp_cache_dir)
        cache.store("test.py", sample_chunks, "hash")
        future_time = datetime.now() + timedelta(hours=1)
        count = cache.invalidate(older_than=future_time)
        assert count == 1
        assert cache.retrieve("test.py") is None

    @classmethod
    def test_cache_statistics(cls, sample_chunks, temp_cache_dir):
        """Test cache statistics."""
        cache = DefaultChunkCache(temp_cache_dir)
        cache.store("test.py", sample_chunks, "hash")
        cache.retrieve("test.py", "hash")
        cache.retrieve("test.py", "wrong_hash")
        cache.retrieve("nonexistent.py")
        stats = cache.get_statistics()
        assert stats["entries"] == 1
        assert stats["hit_rate"] > 0
        assert stats["stats"]["hits"] == 1
        assert stats["stats"]["misses"] == 1
        assert stats["stats"]["hash_mismatches"] == 1

    @classmethod
    def test_export_import_cache(cls, sample_chunks, temp_cache_dir):
        """Test exporting and importing cache."""
        cache1 = DefaultChunkCache(temp_cache_dir + "_1")
        cache2 = DefaultChunkCache(temp_cache_dir + "_2")
        cache1.store("test.py", sample_chunks, "hash")
        export_path = temp_cache_dir + "/export.json"
        cache1.export_cache(export_path)
        cache2.import_cache(export_path)
        entry = cache2.retrieve("test.py")
        assert entry is not None
        assert len(entry.chunks) == len(sample_chunks)


class TestChangeDetector:
    """Test change detector implementation."""

    @classmethod
    def test_compute_file_hash(cls):
        """Test file hash computation."""
        detector = DefaultChangeDetector()
        content = "Hello, World!"
        hash1 = detector.compute_file_hash(content)
        hash2 = detector.compute_file_hash(content)
        hash3 = detector.compute_file_hash(content + " ")
        assert hash1 == hash2
        assert hash1 != hash3

    @classmethod
    def test_find_changed_regions(cls):
        """Test finding changed regions."""
        detector = DefaultChangeDetector()
        old_content = "line 1\nline 2\nline 3\nline 4\nline 5"
        new_content = "line 1\nline 2 modified\nline 3\nnew line\nline 5"
        regions = detector.find_changed_regions(old_content, new_content)
        assert len(regions) > 0
        assert any(r[0] <= 2 <= r[1] for r in regions)

    @classmethod
    def test_classify_change(cls, sample_chunks):
        """Test change classification."""
        detector = DefaultChangeDetector()
        chunk = sample_chunks[0]
        changed_lines = set(range(chunk.start_line, chunk.end_line + 1))
        change_type = detector.classify_change(chunk, "", changed_lines)
        assert change_type == ChangeType.DELETED
        changed_lines = {chunk.start_line}
        change_type = detector.classify_change(chunk, "", changed_lines)
        assert change_type == ChangeType.MODIFIED


class TestIncrementalIndex:
    """Test incremental index implementation."""

    @classmethod
    def test_update_chunk(cls, sample_chunks):
        """Test updating single chunk."""
        index = SimpleIncrementalIndex()
        chunk = sample_chunks[0]
        index.update_chunk(None, chunk)
        assert chunk.chunk_id in index.index
        assert index.index[chunk.chunk_id]["content"] == chunk.content.lower()

    @classmethod
    def test_batch_update(cls, sample_chunks):
        """Test batch update."""
        index = SimpleIncrementalIndex()
        diff = ChunkDiff(
            changes=[
                ChunkChange(
                    chunk_id="chunk1",
                    change_type=ChangeType.MODIFIED,
                    old_chunk=sample_chunks[0],
                    new_chunk=sample_chunks[0],
                    line_changes=[(1, 5)],
                    confidence=0.9,
                ),
            ],
            added_chunks=[],
            deleted_chunks=[],
            modified_chunks=[(sample_chunks[0], sample_chunks[0])],
            unchanged_chunks=sample_chunks[1:],
            summary={"modified": 1},
        )
        index.batch_update(diff)
        assert len(index.update_log) == len(diff.changes)

    @classmethod
    def test_get_update_cost(cls, sample_chunks):
        """Test update cost estimation."""
        index = SimpleIncrementalIndex()
        for chunk in sample_chunks:
            index.update_chunk(None, chunk)
        small_diff = ChunkDiff(
            changes=[
                ChunkChange(
                    chunk_id="chunk1",
                    change_type=ChangeType.MODIFIED,
                    old_chunk=sample_chunks[0],
                    new_chunk=sample_chunks[0],
                    line_changes=[(1, 5)],
                    confidence=0.9,
                ),
            ],
            added_chunks=[],
            deleted_chunks=[],
            modified_chunks=[(sample_chunks[0], sample_chunks[0])],
            unchanged_chunks=sample_chunks[1:],
            summary={"modified": 1, "total": 3},
        )
        small_cost = index.get_update_cost(small_diff)
        assert small_cost < 0.5
        large_diff = ChunkDiff(
            changes=[
                ChunkChange(
                    chunk_id=chunk.chunk_id,
                    change_type=ChangeType.DELETED,
                    old_chunk=chunk,
                    new_chunk=None,
                    line_changes=[(chunk.start_line, chunk.end_line)],
                    confidence=1.0,
                )
                for chunk in sample_chunks
            ],
            added_chunks=[],
            deleted_chunks=sample_chunks,
            modified_chunks=[],
            unchanged_chunks=[],
            summary={"deleted": len(sample_chunks), "total": len(sample_chunks)},
        )
        large_cost = index.get_update_cost(large_diff)
        assert large_cost >= 0.5

    @classmethod
    def test_search(cls, sample_chunks):
        """Test search functionality."""
        index = SimpleIncrementalIndex()
        for chunk in sample_chunks:
            index.update_chunk(None, chunk)
        results = index.search("hello")
        assert len(results) == 1
        assert results[0] == "chunk1"
        results = index.search("print")
        assert len(results) >= 2



"""Semantic analysis and merging module."""

from .analyzer import TreeSitterRelationshipAnalyzer
from .merger import MergeConfig, TreeSitterSemanticMerger

__all__ = [
    "MergeConfig",
    "TreeSitterRelationshipAnalyzer",
    "TreeSitterSemanticMerger",
]



"""Base implementation of symbol resolution.

Provides functionality to find symbol definitions and references in the AST.
"""

from tree_sitter import Node

from chunker.interfaces.context import SymbolResolver


class BaseSymbolResolver(SymbolResolver):
    """Base implementation of symbol resolution with common functionality."""

    def __init__(self, language: str):
        """Initialize the symbol resolver.

        Args:
            language: Language identifier
        """
        self.language = language
        self._definition_cache: dict[str, Node | None] = {}
        self._reference_cache: dict[str, list[Node]] = {}

    def find_symbol_definition(
        self,
        symbol_name: str,
        scope_node: Node,
        ast: Node,
    ) -> Node | None:
        """Find where a symbol is defined.

        Args:
            symbol_name: Name of the symbol to find
            scope_node: Node representing the current scope
            ast: Full AST to search

        Returns:
            Node where symbol is defined, or None
        """
        cache_key = f"{symbol_name}:{id(scope_node)}"
        if cache_key in self._definition_cache:
            return self._definition_cache[cache_key]
        current_scope = scope_node
        while current_scope:
            definition = self._search_scope_for_definition(symbol_name, current_scope)
            if definition:
                self._definition_cache[cache_key] = definition
                return definition
            current_scope = self._get_parent_scope(current_scope)
        definition = self._search_scope_for_definition(symbol_name, ast)
        self._definition_cache[cache_key] = definition
        return definition

    def get_symbol_type(self, symbol_node: Node) -> str:
        """Get the type of a symbol (function, class, variable, etc).

        Args:
            symbol_node: Node representing the symbol

        Returns:
            Type identifier (e.g., 'function', 'class', 'variable')
        """
        parent = symbol_node.parent
        if not parent:
            return "unknown"

        # Check node type mapping
        symbol_type = self._check_type_mapping(parent)
        if symbol_type:
            return symbol_type

        # Check keyword patterns
        return self._check_type_by_keyword(parent.type)

    def _check_type_mapping(self, parent: Node) -> str | None:
        """Check if parent matches known type mappings."""
        node_type_map = self._get_node_type_map()
        parent_type = parent.type

        if parent_type in node_type_map:
            return node_type_map[parent_type]

        if parent.parent and parent.parent.type in node_type_map:
            return node_type_map[parent.parent.type]

        return None

    @staticmethod
    def _check_type_by_keyword(parent_type: str) -> str:
        """Check symbol type by keyword patterns."""
        keyword_patterns = [
            (["function", "method"], "function"),
            (["class"], "class"),
            (["variable", "assignment"], "variable"),
            (["parameter"], "parameter"),
            (["import"], "import"),
        ]

        for keywords, symbol_type in keyword_patterns:
            if any(keyword in parent_type for keyword in keywords):
                return symbol_type

        return "unknown"

    def find_symbol_references(self, symbol_name: str, ast: Node) -> list[Node]:
        """Find all references to a symbol.

        Args:
            symbol_name: Name of the symbol
            ast: AST to search

        Returns:
            List of nodes that reference the symbol
        """
        if symbol_name in self._reference_cache:
            return self._reference_cache[symbol_name]
        references = []

        def find_references(node: Node):
            """Recursively find references to the symbol."""
            if self._is_identifier_node(node):
                name = self._get_node_text(node)
                if name == symbol_name and not self._is_definition_context(
                    node,
                ):
                    references.append(node)
            for child in node.children:
                find_references(child)

        find_references(ast)
        self._reference_cache[symbol_name] = references
        return references

    def _search_scope_for_definition(
        self,
        symbol_name: str,
        scope_node: Node,
    ) -> Node | None:
        """Search within a scope for a symbol definition.

        Args:
            symbol_name: Name to search for
            scope_node: Scope to search within

        Returns:
            Definition node or None
        """

        def search_node(node: Node) -> Node | None:
            if self._is_definition_node(node):
                defined_name = self._get_defined_name(node)
                if defined_name == symbol_name:
                    return node
            for child in node.children:
                if not self._creates_new_scope(child) or child == scope_node:
                    result = search_node(child)
                    if result:
                        return result
            return None

        return search_node(scope_node)

    def _get_parent_scope(self, node: Node) -> Node | None:
        """Get the parent scope of a node.

        Args:
            node: Current node

        Returns:
            Parent scope node or None
        """
        current = node.parent
        while current:
            if self._creates_new_scope(current):
                return current
            current = current.parent
        return None

    @staticmethod
    def _get_node_text(_node: Node) -> str:
        """Get the text content of a node.

        Args:
            node: Node to get text from

        Returns:
            Text content
        """
        return ""

    @staticmethod
    def _get_node_type_map() -> dict[str, str]:
        """Get mapping from AST node types to symbol types.

        Returns:
            Dictionary mapping node types to symbol types
        """
        return {}

    @staticmethod
    def _is_identifier_node(node: Node) -> bool:
        """Check if a node is an identifier.

        Args:
            node: Node to check

        Returns:
            True if node is an identifier
        """
        return node.type == "identifier"

    @staticmethod
    def _is_definition_node(_node: Node) -> bool:
        """Check if a node defines a symbol.

        Args:
            node: Node to check

        Returns:
            True if node defines a symbol
        """
        return False

    @staticmethod
    def _is_definition_context(_node: Node) -> bool:
        """Check if an identifier node is in a definition context.

        Args:
            node: Identifier node

        Returns:
            True if this is a definition, not a reference
        """
        return False

    @staticmethod
    def _get_defined_name(_node: Node) -> str | None:
        """Get the name being defined by a definition node.

        Args:
            node: Definition node

        Returns:
            Name being defined or None
        """
        return None

    @staticmethod
    def _creates_new_scope(_node: Node) -> bool:
        """Check if a node creates a new scope.

        Args:
            node: Node to check

        Returns:
            True if node creates a new scope
        """
        return False



"""Test C++-specific language features for the chunker.

Note: The C++ language plugin (CppPlugin) exists and supports many C++ constructs
including classes, namespaces, templates, constructors/destructors, etc. However,
without a proper LanguageConfig registration (like Python has), the chunker falls
back to default chunk types which only includes function_definition, class_definition,
and method_definition.

As a result, these tests verify that C++ code can be parsed and that functions/methods
are properly detected, but more advanced chunking (classes, namespaces, templates as
separate chunks) requires proper configuration setup.
"""

import importlib

import pytest

import chunker.languages.cpp
from chunker import chunk_file, get_parser
from chunker.exceptions import LanguageNotFoundError, ParserInitError
from chunker.languages import language_config_registry


class TestCppLanguageFeatures:
    """Test C++-specific language features."""

    @staticmethod
    def setup_method():
        """Setup for each test."""
        language_config_registry.clear()
        importlib.reload(chunker.languages.cpp)

    @staticmethod
    def test_cpp_parser_available():
        """Test that C++ parser is available."""
        try:
            parser = get_parser("cpp")
            assert parser is not None
        except (LanguageNotFoundError, ParserInitError):
            pytest.skip("C++ parser not available")

    @staticmethod
    def test_template_function(tmp_path):
        """Test template function chunking."""
        test_file = tmp_path / "template.cpp"
        test_file.write_text(
            """
template<typename T>
T max(T a, T b) {
    return (a > b) ? a : b;
}

template<typename T, typename U>
auto add(T a, U b) -> decltype(a + b) {
    return a + b;
}
""",
        )
        chunks = chunk_file(test_file, "cpp")
        assert len(chunks) == 2
        for chunk in chunks:
            assert chunk.node_type == "function_definition"
        assert any("max" in chunk.content for chunk in chunks)
        assert any("add" in chunk.content for chunk in chunks)

    @staticmethod
    def test_template_class(tmp_path):
        """Test template class chunking."""
        test_file = tmp_path / "template_class.cpp"
        test_file.write_text(
            """
template<typename T>
class Vector {
public:
    Vector() : size_(0), capacity_(0), data_(nullptr) {}

    void push_back(const T& value) {
        // Implementation
    }

    T& operator[](size_t index) {
        return data_[index];
    }

private:
    size_t size_;
    size_t capacity_;
    T* data_;
};

// Template specialization
template<>
class Vector<bool> {
public:
    Vector() : size_(0) {}

    void push_back(bool value) {
        // Bit-packed implementation
    }

private:
    size_t size_;
    std::vector<uint8_t> data_;
};
""",
        )
        chunks = chunk_file(test_file, "cpp")
        assert len(chunks) >= 4
        method_names = [
            c.content.split("(")[0].split()[-1] for c in chunks if "(" in c.content
        ]
        assert "Vector" in method_names
        assert "push_back" in method_names
        assert "operator[]" in method_names or "operator" in " ".join(method_names)

    @staticmethod
    def test_namespace_handling(tmp_path):
        """Test namespace definitions and nested namespaces."""
        test_file = tmp_path / "namespace.cpp"
        test_file.write_text(
            """
namespace MyLib {
    void globalFunction() {
        // Global function in namespace
    }

    class Logger {
    public:
        void log(const std::string& msg) {
            // Implementation
        }
    };

    namespace Utils {
        template<typename T>
        void swap(T& a, T& b) {
            T temp = a;
            a = b;
            b = temp;
        }

        class Helper {
        public:
            static void help() {
                // Implementation
            }
        };
    }
}

namespace MyLib::Network {  // C++17 nested namespace
    class Socket {
    public:
        void connect(const std::string& host, int port) {
            // Implementation
        }
        void disconnect() {
            // Implementation
        }
    };
}
""",
        )
        chunks = chunk_file(test_file, "cpp")
        function_chunks = [c for c in chunks if c.node_type == "function_definition"]
        assert len(function_chunks) >= 5
        assert any("swap" in c.content for c in chunks)
        assert any("log" in c.content for c in chunks)
        assert any("connect" in c.content for c in chunks)

    @staticmethod
    def test_virtual_functions_and_inheritance(tmp_path):
        """Test virtual functions and class inheritance."""
        test_file = tmp_path / "virtual.cpp"
        test_file.write_text(
            """
class Shape {
public:
    virtual ~Shape() = default;

    virtual double area() const = 0;
    virtual double perimeter() const = 0;

    virtual void draw() {
        // Default implementation
    }
};

class Circle : public Shape {
public:
    Circle(double radius) : radius_(radius) {}

    double area() const override {
        return 3.14159 * radius_ * radius_;
    }

    double perimeter() const override {
        return 2 * 3.14159 * radius_;
    }

    void draw() override {
        // Circle-specific drawing
    }

private:
    double radius_;
};

class Rectangle final : public Shape {
public:
    Rectangle(double width, double height)
        : width_(width), height_(height) {}

    double area() const override {
        return width_ * height_;
    }

    double perimeter() const override {
        return 2 * (width_ + height_);
    }

private:
    double width_;
    double height_;
};
""",
        )
        chunks = chunk_file(test_file, "cpp")
        function_chunks = [c for c in chunks if c.node_type == "function_definition"]
        assert len(function_chunks) >= 8
        virtual_methods = [
            c for c in chunks if "virtual" in c.content or "override" in c.content
        ]
        assert len(virtual_methods) >= 6
        assert any("area" in c.content for c in chunks)
        assert any("perimeter" in c.content for c in chunks)

    @staticmethod
    def test_operator_overloading(tmp_path):
        """Test operator overloading detection."""
        test_file = tmp_path / "operators.cpp"
        test_file.write_text(
            """
class Complex {
public:
    Complex(double real = 0, double imag = 0)
        : real_(real), imag_(imag) {}

    // Arithmetic operators
    Complex operator+(const Complex& other) const {
        return Complex(real_ + other.real_, imag_ + other.imag_);
    }

    Complex operator-(const Complex& other) const {
        return Complex(real_ - other.real_, imag_ - other.imag_);
    }

    Complex& operator+=(const Complex& other) {
        real_ += other.real_;
        imag_ += other.imag_;
        return *this;
    }

    // Comparison operators
    bool operator==(const Complex& other) const {
        return real_ == other.real_ && imag_ == other.imag_;
    }

    bool operator!=(const Complex& other) const {
        return !(*this == other);
    }

    // Stream operators (friend functions)
    friend std::ostream& operator<<(std::ostream& os, const Complex& c) {
        os << c.real_ << " + " << c.imag_ << "i";
        return os;
    }

    // Conversion operator
    operator double() const {
        return std::sqrt(real_ * real_ + imag_ * imag_);
    }

    // Function call operator
    double operator()() const {
        return real_ * real_ + imag_ * imag_;
    }

    // Subscript operator
    double& operator[](int index) {
        return (index == 0) ? real_ : imag_;
    }

private:
    double real_;
    double imag_;
};

// Global operator
Complex operator*(double scalar, const Complex& c) {
    return Complex(scalar * c.real_, scalar * c.imag_);
}
""",
        )
        chunks = chunk_file(test_file, "cpp")
        operator_chunks = [c for c in chunks if "operator" in c.content]
        assert len(operator_chunks) >= 10
        assert any("operator+" in c.content for c in operator_chunks)
        assert any("operator==" in c.content for c in operator_chunks)
        assert any("operator<<" in c.content for c in operator_chunks)
        assert any("operator()" in c.content for c in operator_chunks)
        assert any("operator[]" in c.content for c in operator_chunks)

    @staticmethod
    def test_stl_usage_patterns(tmp_path):
        """Test STL container and algorithm usage."""
        test_file = tmp_path / "stl_usage.cpp"
        test_file.write_text(
            """
#include <vector>
#include <map>
#include <algorithm>
#include <memory>
#include <functional>

class DataProcessor {
public:
    using DataMap = std::map<std::string, std::vector<int>>;
    using ProcessFunc = std::function<void(int&)>;

    void addData(const std::string& key, std::initializer_list<int> values) {
        data_[key] = std::vector<int>(values);
    }

    void processAll(ProcessFunc func) {
        for (auto& [key, values] : data_) {
            std::for_each(values.begin(), values.end(), func);
        }
    }

    std::vector<int> getSorted(const std::string& key) {
        auto it = data_.find(key);
        if (it != data_.end()) {
            std::vector<int> sorted = it->second;
            std::sort(sorted.begin(), sorted.end());
            return sorted;
        }
        return {};
    }

    template<typename Predicate>
    std::vector<int> filter(const std::string& key, Predicate pred) {
        std::vector<int> result;
        auto it = data_.find(key);
        if (it != data_.end()) {
            std::copy_if(it->second.begin(), it->second.end(),
                        std::back_inserter(result), pred);
        }
        return result;
    }

private:
    DataMap data_;
    std::unique_ptr<ProcessFunc> custom_processor_;
};

// Function using modern C++ features
auto makeProcessor() -> std::unique_ptr<DataProcessor> {
    return std::make_unique<DataProcessor>();
}

// Lambda and STL algorithms
void demonstrateSTL() {
    std::vector<int> nums = {1, 2, 3, 4, 5};

    // Lambda with capture
    int sum = 0;
    std::for_each(nums.begin(), nums.end(), [&sum](int n) {
        sum += n;
    });

    // Transform with lambda
    std::vector<int> squares;
    std::transform(nums.begin(), nums.end(), std::back_inserter(squares),
                  [](int n) { return n * n; });
}
""",
        )
        chunks = chunk_file(test_file, "cpp")
        function_chunks = [c for c in chunks if c.node_type == "function_definition"]
        assert len(function_chunks) >= 6
        template_methods = [c for c in chunks if "filter" in c.content]
        assert len(template_methods) >= 1
        auto_functions = [c for c in chunks if "makeProcessor" in c.content]
        assert len(auto_functions) >= 1
        stl_demo = [c for c in chunks if "demonstrateSTL" in c.content]
        assert len(stl_demo) >= 1

    @staticmethod
    def test_constructor_destructor(tmp_path):
        """Test constructor and destructor detection."""
        test_file = tmp_path / "ctor_dtor.cpp"
        test_file.write_text(
            """
class Resource {
public:
    // Default constructor
    Resource() : data_(nullptr), size_(0) {
        std::cout << "Default constructor\\n";
    }

    // Parameterized constructor
    Resource(size_t size) : size_(size) {
        data_ = new int[size];
        std::cout << "Parameterized constructor\\n";
    }

    // Copy constructor
    Resource(const Resource& other) : size_(other.size_) {
        data_ = new int[size_];
        std::copy(other.data_, other.data_ + size_, data_);
        std::cout << "Copy constructor\\n";
    }

    // Move constructor
    Resource(Resource&& other) noexcept
        : data_(other.data_), size_(other.size_) {
        other.data_ = nullptr;
        other.size_ = 0;
        std::cout << "Move constructor\\n";
    }

    // Destructor
    ~Resource() {
        delete[] data_;
        std::cout << "Destructor\\n";
    }

    // Copy assignment
    Resource& operator=(const Resource& other) {
        if (this != &other) {
            delete[] data_;
            size_ = other.size_;
            data_ = new int[size_];
            std::copy(other.data_, other.data_ + size_, data_);
        }
        return *this;
    }

    // Move assignment
    Resource& operator=(Resource&& other) noexcept {
        if (this != &other) {
            delete[] data_;
            data_ = other.data_;
            size_ = other.size_;
            other.data_ = nullptr;
            other.size_ = 0;
        }
        return *this;
    }

private:
    int* data_;
    size_t size_;
};
""",
        )
        chunks = chunk_file(test_file, "cpp")
        all_functions = [c for c in chunks if c.node_type == "function_definition"]
        assert len(all_functions) >= 7
        constructor_like = [c for c in chunks if "Resource(" in c.content]
        assert len(constructor_like) >= 3
        destructor_like = [c for c in chunks if "~Resource" in c.content]
        assert len(destructor_like) >= 1
        assignment_chunks = [c for c in chunks if "operator=" in c.content]
        assert len(assignment_chunks) == 2

    @staticmethod
    def test_cpp_specific_features(tmp_path):
        """Test various C++-specific features."""
        test_file = tmp_path / "cpp_features.cpp"
        test_file.write_text(
            """
// Inline namespace
inline namespace v1 {
    class Version1 {
    public:
        void feature() { /* v1 implementation */ }
    };
}

// Constexpr function
constexpr int factorial(int n) {
    return (n <= 1) ? 1 : n * factorial(n - 1);
}

// Variadic template
template<typename... Args>
void print(Args... args) {
    ((std::cout << args << " "), ...);
}

// Enum class
enum class Color : uint8_t {
    Red = 0,
    Green = 1,
    Blue = 2
};

// Structured binding example
struct Point {
    double x, y, z;
};

Point getPoint() {
    return {1.0, 2.0, 3.0};
}

// Using declaration
using ColorType = std::underlying_type_t<Color>;

// Concept (C++20)
template<typename T>
concept Arithmetic = std::is_arithmetic_v<T>;

template<Arithmetic T>
T add(T a, T b) {
    return a + b;
}

// Class with deleted functions
class NonCopyable {
public:
    NonCopyable() = default;
    NonCopyable(const NonCopyable&) = delete;
    NonCopyable& operator=(const NonCopyable&) = delete;
    NonCopyable(NonCopyable&&) = default;
    NonCopyable& operator=(NonCopyable&&) = default;
};
""",
        )
        chunks = chunk_file(test_file, "cpp")
        assert len(chunks) >= 4
        constexpr_chunks = [
            c for c in chunks if "constexpr" in c.content and "factorial" in c.content
        ]
        assert len(constexpr_chunks) >= 1
        variadic_chunks = [
            c for c in chunks if "print" in c.content and "..." in c.content
        ]
        assert len(variadic_chunks) >= 1
        concept_chunks = [
            c
            for c in chunks
            if "add" in c.content and c.node_type == "function_definition"
        ]
        assert len(concept_chunks) >= 1
        assert any("getPoint" in c.content for c in chunks)

    @staticmethod
    def test_nested_classes_and_structs(tmp_path):
        """Test nested classes and structs."""
        test_file = tmp_path / "nested.cpp"
        test_file.write_text(
            """
class OuterClass {
public:
    class InnerClass {
    public:
        void innerMethod() {
            // Inner class method
        }

        struct InnerStruct {
            int value;
            void structMethod() {
                // Struct method
            }
        };
    };

    struct OuterStruct {
        void outerStructMethod() {
            // Outer struct method
        }
    };

private:
    class PrivateNested {
        void privateMethod() {
            // Private nested class
        }
    };
};

// Test struct with nested types
struct Container {
    struct Node {
        int data;
        Node* next;

        Node(int val) : data(val), next(nullptr) {}
    };

    class Iterator {
    public:
        Iterator(Node* node) : current_(node) {}

        Iterator& operator++() {
            current_ = current_->next;
            return *this;
        }

    private:
        Node* current_;
    };
};
""",
        )
        chunks = chunk_file(test_file, "cpp")
        function_chunks = [c for c in chunks if c.node_type == "function_definition"]
        assert len(function_chunks) >= 5
        inner_method_chunks = [c for c in chunks if "innerMethod" in c.content]
        assert len(inner_method_chunks) >= 1
        assert any("structMethod" in c.content for c in chunks)
        assert any("operator++" in c.content for c in chunks)



"""
Unit tests for distribution implementation
"""

import tempfile
from pathlib import Path
from unittest.mock import MagicMock, patch

from chunker.distribution.manager import DistributionImpl
from chunker.distribution.release import ReleaseManagementImpl


class TestDistributionImpl:
    """Test the distribution implementation"""

    @classmethod
    def test_publish_to_pypi_missing_directory(cls):
        """Test publish fails with missing directory"""
        dist = DistributionImpl()
        success, info = dist.publish_to_pypi(Path("/nonexistent"))
        assert success is False
        assert "Package directory not found" in info["errors"][0]
        assert info["status"] == "failed"

    @classmethod
    def test_publish_to_pypi_no_files(cls):
        """Test publish fails with no distribution files"""
        dist = DistributionImpl()
        with tempfile.TemporaryDirectory() as tmpdir:
            success, info = dist.publish_to_pypi(Path(tmpdir))
            assert success is False
            assert "No distribution files found" in info["errors"][0]
            assert info["status"] == "failed"

    @classmethod
    @patch("subprocess.run")
    def test_publish_to_pypi_dry_run(cls, mock_run):
        """Test dry run publish"""
        dist = DistributionImpl()
        mock_run.return_value = MagicMock(returncode=0, stdout="", stderr="")
        with tempfile.TemporaryDirectory() as tmpdir:
            tmppath = Path(tmpdir)
            wheel = tmppath / "test-1.0.0-py3-none-any.whl"
            wheel.write_text("dummy wheel")
            success, info = dist.publish_to_pypi(tmppath, dry_run=True)
            assert success is True
            assert info["status"] == "dry_run_success"
            assert len(info["uploaded"]) == 1
            assert info["uploaded"][0]["status"] == "would_upload"

    @classmethod
    @patch("subprocess.run")
    def test_build_docker_image_success(cls, mock_run):
        """Test successful Docker image build"""
        dist = DistributionImpl()
        mock_run.side_effect = [
            MagicMock(returncode=0),
            MagicMock(returncode=0),
            MagicMock(returncode=0, stdout="abc123\n"),
        ]
        success, image_id = dist.build_docker_image("test:latest")
        assert success is True
        assert image_id == "abc123"

    @classmethod
    @patch("subprocess.run")
    def test_build_docker_image_no_docker(cls, mock_run):
        """Test Docker build when Docker is not available"""
        dist = DistributionImpl()
        mock_run.side_effect = FileNotFoundError()
        success, image_id = dist.build_docker_image("test:latest")
        assert success is False
        assert image_id == "docker-not-available"

    @classmethod
    def test_create_homebrew_formula(cls):
        """Test Homebrew formula generation"""
        dist = DistributionImpl()
        with tempfile.TemporaryDirectory() as tmpdir:
            output_path = Path(tmpdir)
            success, formula_path = dist.create_homebrew_formula("1.0.0", output_path)
            assert success is True
            assert formula_path.exists()
            assert formula_path.name == "treesitter-chunker.rb"
            content = formula_path.read_text()
            assert "class TreesitterChunker < Formula" in content
            assert (
                'version "1.0.0"' in content
                or "treesitter-chunker-1.0.0.tar.gz" in content
            )

    @classmethod
    @patch("subprocess.run")
    def test_verify_installation_pip(cls, mock_run):
        """Test pip installation verification"""
        dist = DistributionImpl()
        mock_run.side_effect = [
            MagicMock(returncode=0),
            MagicMock(returncode=0),
            MagicMock(returncode=0, stdout="1.0.0"),
        ]
        success, details = dist.verify_installation("pip", "linux")
        assert success is True
        assert details["installed"] is True
        assert details["functional"] is True
        assert details["version"] == "1.0.0"

    @classmethod
    @patch("subprocess.run")
    def test_verify_installation_docker(cls, mock_run):
        """Test Docker installation verification"""
        dist = DistributionImpl()
        mock_run.return_value = MagicMock(returncode=0)
        success, details = dist.verify_installation("docker", "linux")
        assert success is True
        assert details["installed"] is True
        assert details["functional"] is True


class TestReleaseManagementImpl:
    """Test the release management implementation"""

    @classmethod
    def test_prepare_release_invalid_version(cls):
        """Test prepare release with invalid version format"""
        release = ReleaseManagementImpl()
        success, info = release.prepare_release("invalid", "changelog")
        assert success is False
        assert "Invalid version format" in info["errors"][0]
        assert info["status"] == "failed"

    @classmethod
    @patch("subprocess.run")
    def test_prepare_release_success(cls, mock_run):
        """Test successful release preparation"""
        with tempfile.TemporaryDirectory() as tmpdir:
            tmppath = Path(tmpdir)
            release = ReleaseManagementImpl(tmppath)
            setup_py = tmppath / "setup.py"
            setup_py.write_text('version="0.1.0"')
            changelog = tmppath / "CHANGELOG.md"
            changelog.write_text("# Changelog\n\n")
            mock_run.side_effect = [
                MagicMock(returncode=0),
                MagicMock(returncode=0, stdout=""),
                MagicMock(returncode=0),
            ]
            success, info = release.prepare_release("1.0.0", "New features")
            assert success is True
            assert info["status"] == "success"
            assert str(setup_py) in info["files_updated"]
            assert str(changelog) in info["files_updated"]
            assert 'version="1.0.0"' in setup_py.read_text()
            assert "[1.0.0]" in changelog.read_text()
            assert "New features" in changelog.read_text()

    @classmethod
    @patch("subprocess.run")
    def test_create_release_artifacts(cls, mock_run):
        """Test release artifact creation"""
        with tempfile.TemporaryDirectory() as tmpdir:
            tmppath = Path(tmpdir)
            release = ReleaseManagementImpl(tmppath)
            output_dir = tmppath / "dist"
            output_dir.mkdir()
            sdist = output_dir / "treesitter-chunker-1.0.0.tar.gz"
            wheel = output_dir / "treesitter_chunker-1.0.0-py3-none-any.whl"

            def create_artifacts(*args, **kwargs):
                if "--sdist" in args[0]:
                    sdist.write_text("sdist content")
                elif "--wheel" in args[0]:
                    wheel.write_text("wheel content")
                return MagicMock(returncode=0, stdout="", stderr="")

            mock_run.side_effect = create_artifacts
            artifacts = release.create_release_artifacts("1.0.0", output_dir)
            assert len(artifacts) >= 2
            checksum_files = [a for a in artifacts if a.name.endswith(".sha256")]
            assert len(checksum_files) == 1
            notes_files = [a for a in artifacts if a.name.startswith("RELEASE_NOTES")]
            assert len(notes_files) == 1

    @classmethod
    def test_version_comparison(cls):
        """Test version comparison logic"""
        release = ReleaseManagementImpl()
        assert release._is_version_higher("1.0.0", "0.9.0") is True
        assert release._is_version_higher("1.0.1", "1.0.0") is True
        assert release._is_version_higher("2.0.0", "1.9.9") is True
        assert release._is_version_higher("1.0.0", "1.0.0") is False
        assert release._is_version_higher("0.9.0", "1.0.0") is False
        assert release._is_version_higher("1.0.0", "1.0.1") is False



"""User-friendly tools for managing tree-sitter grammars."""

import json
import logging
import os
import shutil
import subprocess
from pathlib import Path
from typing import Any, Dict, List, Optional, Tuple
from urllib.parse import urlparse

from .grammar_management import GrammarCompatibility, GrammarHealth, SmartGrammarManager

logger = logging.getLogger(__name__)


class UserGrammarTools:
    """User-friendly tools for grammar management."""

    def __init__(self, build_dir: Path, grammars_dir: Path):
        """Initialize user grammar tools.

        Args:
            build_dir: Directory containing compiled .so files
            grammars_dir: Directory containing grammar source repositories
        """
        self.build_dir = Path(build_dir)
        self.grammars_dir = Path(grammars_dir)
        self.manager = SmartGrammarManager(build_dir, grammars_dir)

    def install_grammar(
        self,
        language: str,
        repo_url: str,
        branch: str = "main",
    ) -> dict[str, Any]:
        """Install a grammar from a repository.

        Args:
            language: Language name
            repo_url: Git repository URL
            branch: Branch to checkout

        Returns:
            Installation result with status and details
        """
        result = {
            "language": language,
            "status": "unknown",
            "steps_completed": [],
            "errors": [],
            "warnings": [],
        }

        try:
            # Step 1: Clone repository
            target_dir = self.grammars_dir / f"tree-sitter-{language}"
            if target_dir.exists():
                result["warnings"].append(f"Directory {target_dir} already exists")
                # Try to update instead
                try:
                    subprocess.run(
                        ["git", "fetch", "origin"],
                        cwd=target_dir,
                        check=True,
                        capture_output=True,
                        text=True,
                    )
                    subprocess.run(
                        ["git", "checkout", branch],
                        cwd=target_dir,
                        check=True,
                        capture_output=True,
                        text=True,
                    )
                    result["steps_completed"].append("Updated existing repository")
                except subprocess.CalledProcessError as e:
                    result["errors"].append(f"Failed to update repository: {e}")
                    result["status"] = "error"
                    return result
            else:
                try:
                    subprocess.run(
                        ["git", "clone", "--branch", branch, repo_url, str(target_dir)],
                        check=True,
                        capture_output=True,
                        text=True,
                    )
                    result["steps_completed"].append("Cloned repository")
                except subprocess.CalledProcessError as e:
                    result["errors"].append(f"Failed to clone repository: {e}")
                    result["status"] = "error"
                    return result

            # Step 2: Install dependencies
            package_json = target_dir / "package.json"
            if package_json.exists():
                try:
                    subprocess.run(
                        ["npm", "install"],
                        cwd=target_dir,
                        check=True,
                        capture_output=True,
                        text=True,
                    )
                    result["steps_completed"].append("Installed npm dependencies")
                except subprocess.CalledProcessError as e:
                    result["warnings"].append(
                        f"Failed to install npm dependencies: {e}",
                    )

            # Step 3: Generate grammar
            try:
                subprocess.run(
                    ["tree-sitter", "generate"],
                    cwd=target_dir,
                    check=True,
                    capture_output=True,
                    text=True,
                )
                result["steps_completed"].append("Generated grammar")
            except subprocess.CalledProcessError as e:
                result["errors"].append(f"Failed to generate grammar: {e}")
                result["status"] = "error"
                return result

            # Step 4: Copy .so file to build directory
            so_files = list(target_dir.glob("*.so"))
            if so_files:
                for so_file in so_files:
                    target_so = self.build_dir / f"{language}.so"
                    shutil.copy2(so_file, target_so)
                    result["steps_completed"].append(
                        f"Copied {so_file.name} to build directory",
                    )
            else:
                result["errors"].append("No .so files found after generation")
                result["status"] = "error"
                return result

            # Step 5: Validate installation
            health = self.manager.diagnose_grammar_issues(language)
            if health.status == "healthy":
                result["status"] = "success"
                result["steps_completed"].append("Validated grammar installation")
            else:
                result["status"] = "warning"
                result["warnings"].append(
                    f"Grammar installed but has issues: {health.status}",
                )

        except Exception as e:
            result["status"] = "error"
            result["errors"].append(f"Unexpected error: {e}")

        return result

    def remove_grammar(self, language: str) -> dict[str, Any]:
        """Remove a grammar and its source.

        Args:
            language: Language name to remove

        Returns:
            Removal result with status and details
        """
        result = {
            "language": language,
            "status": "unknown",
            "steps_completed": [],
            "errors": [],
            "warnings": [],
        }

        try:
            # Remove .so file
            so_file = self.build_dir / f"{language}.so"
            if so_file.exists():
                so_file.unlink()
                result["steps_completed"].append("Removed compiled grammar library")
            else:
                result["warnings"].append("No compiled grammar library found")

            # Remove source directory
            source_dir = self.grammars_dir / f"tree-sitter-{language}"
            if source_dir.exists():
                shutil.rmtree(source_dir)
                result["steps_completed"].append("Removed grammar source directory")
            else:
                result["warnings"].append("No grammar source directory found")

            result["status"] = "success"

        except Exception as e:
            result["status"] = "error"
            result["errors"].append(f"Failed to remove grammar: {e}")

        return result

    def update_grammar(self, language: str) -> dict[str, Any]:
        """Update a grammar to the latest version.

        Args:
            language: Language name to update

        Returns:
            Update result with status and details
        """
        result = {
            "language": language,
            "status": "unknown",
            "steps_completed": [],
            "errors": [],
            "warnings": [],
        }

        try:
            source_dir = self.grammars_dir / f"tree-sitter-{language}"
            if not source_dir.exists():
                result["errors"].append(f"Grammar source not found: {source_dir}")
                return result

            # Update repository
            try:
                subprocess.run(
                    ["git", "fetch", "origin"],
                    cwd=source_dir,
                    check=True,
                    capture_output=True,
                    text=True,
                )
                result["steps_completed"].append("Fetched latest changes")

                # Get current and latest commit
                current = subprocess.run(
                    ["git", "rev-parse", "HEAD"],
                    cwd=source_dir,
                    capture_output=True,
                    text=True,
                    check=True,
                ).stdout.strip()

                latest = subprocess.run(
                    ["git", "rev-parse", "origin/main"],
                    cwd=source_dir,
                    capture_output=True,
                    text=True,
                    check=True,
                ).stdout.strip()

                if current == latest:
                    result["warnings"].append("Grammar is already up to date")
                    result["status"] = "success"
                    return result

                # Checkout latest
                subprocess.run(
                    ["git", "checkout", "origin/main"],
                    cwd=source_dir,
                    check=True,
                    capture_output=True,
                    text=True,
                )
                result["steps_completed"].append("Updated to latest version")

            except subprocess.CalledProcessError as e:
                result["errors"].append(f"Failed to update repository: {e}")
                result["status"] = "error"
                return result

            # Reinstall dependencies if needed
            package_json = source_dir / "package.json"
            if package_json.exists():
                try:
                    subprocess.run(
                        ["npm", "install"],
                        cwd=source_dir,
                        check=True,
                        capture_output=True,
                        text=True,
                    )
                    result["steps_completed"].append("Updated dependencies")
                except subprocess.CalledProcessError as e:
                    result["warnings"].append(f"Failed to update dependencies: {e}")

            # Regenerate grammar
            try:
                subprocess.run(
                    ["tree-sitter", "generate"],
                    cwd=source_dir,
                    check=True,
                    capture_output=True,
                    text=True,
                )
                result["steps_completed"].append("Regenerated grammar")
            except subprocess.CalledProcessError as e:
                result["errors"].append(f"Failed to regenerate grammar: {e}")
                return result

            # Copy new .so file
            so_files = list(source_dir.glob("*.so"))
            if so_files:
                for so_file in so_files:
                    target_so = self.build_dir / f"{language}.so"
                    shutil.copy2(so_file, target_so)
                    result["steps_completed"].append("Updated compiled grammar library")
            else:
                result["errors"].append("No .so files found after regeneration")
                return result

            result["status"] = "success"

        except Exception as e:
            result["status"] = "error"
            result["errors"].append(f"Unexpected error during update: {e}")

        return result

    def list_installed_grammars(self) -> dict[str, Any]:
        """List all installed grammars with their status.

        Returns:
            Dictionary with grammar information
        """
        result = {
            "total_grammars": 0,
            "healthy_grammars": 0,
            "problematic_grammars": 0,
            "grammars": {},
        }

        # Get all .so files
        so_files = list(self.build_dir.glob("*.so"))
        result["total_grammars"] = len(so_files)

        for so_file in so_files:
            language = so_file.stem
            health = self.manager.diagnose_grammar_issues(language)
            compatibility = self.manager.get_grammar_compatibility(language)

            grammar_info = {
                "status": health.status,
                "issues": health.issues,
                "recommendations": health.recommendations,
                "compatibility_score": compatibility.compatibility_score,
                "compilation_date": compatibility.compilation_date,
                "file_size": so_file.stat().st_size if so_file.exists() else 0,
            }

            result["grammars"][language] = grammar_info

            if health.status == "healthy":
                result["healthy_grammars"] += 1
            else:
                result["problematic_grammars"] += 1

        return result

    def get_grammar_info(self, language: str) -> dict[str, Any]:
        """Get detailed information about a specific grammar.

        Args:
            language: Language name

        Returns:
            Detailed grammar information
        """
        result = {
            "language": language,
            "health": None,
            "compatibility": None,
            "recovery_plan": None,
            "source_info": None,
        }

        # Get health status
        result["health"] = self.manager.diagnose_grammar_issues(language)

        # Get compatibility info
        result["compatibility"] = self.manager.get_grammar_compatibility(language)

        # Get recovery plan if needed
        if result["health"].status != "healthy":
            result["recovery_plan"] = self.manager.generate_recovery_plan(language)

        # Get source repository info
        source_dir = self.grammars_dir / f"tree-sitter-{language}"
        if source_dir.exists():
            try:
                # Get git info
                git_log = subprocess.run(
                    ["git", "log", "--oneline", "-1"],
                    cwd=source_dir,
                    capture_output=True,
                    text=True,
                    check=False,
                )

                git_remote = subprocess.run(
                    ["git", "remote", "get-url", "origin"],
                    cwd=source_dir,
                    capture_output=True,
                    text=True,
                    check=False,
                )

                result["source_info"] = {
                    "repository_url": (
                        git_remote.stdout.strip()
                        if git_remote.returncode == 0
                        else "unknown"
                    ),
                    "latest_commit": (
                        git_log.stdout.strip() if git_log.returncode == 0 else "unknown"
                    ),
                    "source_directory": str(source_dir),
                    "has_package_json": (source_dir / "package.json").exists(),
                    "has_grammar_js": (source_dir / "grammar.js").exists(),
                }
            except Exception:
                result["source_info"] = {
                    "error": "Could not retrieve source information",
                }

        return result

    def check_system_health(self) -> dict[str, Any]:
        """Check overall system health for grammar management.

        Returns:
            System health report
        """
        result = {
            "system_requirements": self.manager.get_system_requirements(),
            "directory_permissions": {},
            "recommendations": [],
        }

        # Check directory permissions
        for dir_path, dir_name in [
            (self.build_dir, "build directory"),
            (self.grammars_dir, "grammars directory"),
        ]:
            try:
                if dir_path.exists():
                    result["directory_permissions"][dir_name] = {
                        "exists": True,
                        "readable": os.access(dir_path, os.R_OK),
                        "writable": os.access(dir_path, os.W_OK),
                        "executable": os.access(dir_path, os.X_OK),
                    }
                else:
                    result["directory_permissions"][dir_name] = {"exists": False}
            except Exception as e:
                result["directory_permissions"][dir_name] = {"error": str(e)}

        # Generate recommendations
        requirements = result["system_requirements"]

        if not requirements["tree_sitter_cli"]:
            result["recommendations"].append(
                "Install tree-sitter CLI: npm install -g tree-sitter-cli",
            )

        if not requirements["node_npm"]:
            result["recommendations"].append(
                "Install Node.js and npm for grammar compilation",
            )

        if not requirements["git"]:
            result["recommendations"].append(
                "Install git for managing grammar repositories",
            )

        if not requirements["compiler"]:
            result["recommendations"].append(
                "Install a C compiler (gcc or clang) for building grammars",
            )

        if not requirements["python_deps"]:
            result["recommendations"].append(
                "Install Python tree-sitter package: pip install tree-sitter",
            )

        return result



"""
Comprehensive unit tests for integration_tester.py

This module provides extensive test coverage for the integration testing framework,
ensuring all methods and edge cases are properly tested with 95%+ coverage.
"""

import json
import tempfile
import time
import unittest
from dataclasses import dataclass
from pathlib import Path
from typing import Any, Dict, List
from unittest.mock import MagicMock, Mock, patch

from ..core.extraction_framework import BaseExtractor, CallSite, ExtractionResult
from .integration_tester import (
    ExtractorTestSuite,
    IntegrationTester,
    TestResult,
    _generate_overall_recommendations,
    _generate_overall_summary,
    run_complete_extractor_test_suite,
)


class MockExtractor(BaseExtractor):
    """Mock extractor for testing."""

    def __init__(self, language: str = "test", should_fail: bool = False):
        super().__init__(language)
        self.should_fail = should_fail
        self._call_count = 0

    def extract_calls(
        self,
        source_code: str,
        file_path: Path | None = None,
    ) -> ExtractionResult:
        """Mock extract_calls method."""
        self._call_count += 1

        if self.should_fail:
            raise Exception("Mock extractor failure")

        result = ExtractionResult()

        # Create mock call sites
        if source_code.strip():
            result.call_sites = [
                CallSite(
                    function_name="mock_function",
                    line_number=1,
                    column_number=0,
                    byte_start=0,
                    byte_end=len(source_code),
                    call_type="function",
                    context={"test": True},
                    language=self.language,
                    file_path=file_path or Path("test.txt"),
                ),
            ]

        result.extraction_time = 0.001
        return result

    def validate_source(self, source_code: str) -> bool:
        """Mock validate_source method."""
        return isinstance(source_code, str) and len(source_code.strip()) > 0


class TestTestResult(unittest.TestCase):
    """Test cases for TestResult class."""

    def test_initialization(self):
        """Test TestResult initialization."""
        result = TestResult("test_name", "python")

        self.assertEqual(result.test_name, "test_name")
        self.assertEqual(result.language, "python")
        self.assertFalse(result.passed)
        self.assertEqual(result.errors, [])
        self.assertEqual(result.warnings, [])
        self.assertEqual(result.execution_time, 0.0)
        self.assertEqual(result.call_sites_found, 0)
        self.assertEqual(result.accuracy_score, 0.0)
        self.assertEqual(result.performance_metrics, {})
        self.assertEqual(result.metadata, {})

    def test_add_error(self):
        """Test adding errors to TestResult."""
        result = TestResult("test", "python")

        # Test simple error
        result.add_error("Test error")
        self.assertEqual(len(result.errors), 1)
        self.assertIn("Test error", result.errors[0])
        self.assertFalse(result.passed)

        # Test error with exception
        exception = ValueError("Test exception")
        result.add_error("Error with exception", exception)
        self.assertEqual(len(result.errors), 2)
        self.assertIn("Error with exception", result.errors[1])
        self.assertIn("Test exception", result.errors[1])

    def test_add_warning(self):
        """Test adding warnings to TestResult."""
        result = TestResult("test", "python")

        result.add_warning("Test warning")
        self.assertEqual(len(result.warnings), 1)
        self.assertEqual(result.warnings[0], "Test warning")

    def test_to_dict(self):
        """Test TestResult serialization to dict."""
        result = TestResult("test_name", "python")
        result.passed = True
        result.accuracy_score = 0.85
        result.call_sites_found = 5
        result.execution_time = 1.23

        result_dict = result.to_dict()

        self.assertEqual(result_dict["test_name"], "test_name")
        self.assertEqual(result_dict["language"], "python")
        self.assertTrue(result_dict["passed"])
        self.assertEqual(result_dict["accuracy_score"], 0.85)
        self.assertEqual(result_dict["call_sites_found"], 5)
        self.assertEqual(result_dict["execution_time"], 1.23)


class TestExtractorTestSuite(unittest.TestCase):
    """Test cases for ExtractorTestSuite class."""

    def setUp(self):
        """Set up test fixtures."""
        self.test_suite = ExtractorTestSuite()

    def test_initialization(self):
        """Test ExtractorTestSuite initialization."""
        # Should have all expected extractors
        expected_extractors = ["python", "javascript", "rust", "go", "c", "cpp", "java"]
        self.assertEqual(
            set(self.test_suite.extractors.keys()),
            set(expected_extractors),
        )

        # Should have test samples for all languages
        self.assertEqual(
            set(self.test_suite.test_samples.keys()),
            set(expected_extractors),
        )

        # Should have performance thresholds
        self.assertEqual(
            set(self.test_suite.performance_thresholds.keys()),
            set(expected_extractors),
        )

    @patch("chunker.extractors.testing.integration_tester.time.perf_counter")
    def test_run_all_tests(self, mock_time):
        """Test running all tests."""
        mock_time.side_effect = [0.0, 1.0, 2.0, 3.0]  # Mock timing

        # Replace extractors with mocks for faster testing
        self.test_suite.extractors = {
            "python": MockExtractor("python"),
            "javascript": MockExtractor("javascript"),
        }

        results = self.test_suite.run_all_tests()

        self.assertIn("overall_passed", results)
        self.assertIn("total_tests", results)
        self.assertIn("passed_tests", results)
        self.assertIn("failed_tests", results)
        self.assertIn("execution_time", results)
        self.assertIn("language_results", results)
        self.assertIn("summary", results)

    def test_test_language_extractors(self):
        """Test language extractor testing."""
        # Use mock extractor for faster testing
        self.test_suite.extractors = {"python": MockExtractor("python")}

        results = self.test_suite.test_language_extractors()

        self.assertIn("python", results)
        python_results = results["python"]

        self.assertIn("extractor_class", python_results)
        self.assertIn("tests", python_results)
        self.assertIn("overall_passed", python_results)
        self.assertIn("total_time", python_results)

    def test_test_cross_language_integration(self):
        """Test cross-language integration testing."""
        # Use mock extractors
        self.test_suite.extractors = {
            "python": MockExtractor("python"),
            "javascript": MockExtractor("javascript"),
        }

        results = self.test_suite.test_cross_language_integration()

        self.assertIn("consistency_tests", results)
        self.assertIn("multi_file_tests", results)
        self.assertIn("performance_comparison", results)
        self.assertIn("overall_passed", results)
        self.assertIn("execution_time", results)

    def test_benchmark_performance(self):
        """Test performance benchmarking."""
        # Use mock extractors for faster testing
        self.test_suite.extractors = {"python": MockExtractor("python")}

        results = self.test_suite.benchmark_performance()

        self.assertIn("language_benchmarks", results)
        self.assertIn("comparative_analysis", results)
        self.assertIn("performance_regression_check", results)
        self.assertIn("resource_usage", results)
        self.assertIn("execution_time", results)

    def test_basic_extraction_test(self):
        """Test basic extraction test method."""
        mock_extractor = MockExtractor("python")

        result = self.test_suite._test_basic_extraction("python", mock_extractor)

        self.assertIsInstance(result, TestResult)
        self.assertEqual(result.test_name, "basic_extraction")
        self.assertEqual(result.language, "python")
        self.assertGreaterEqual(result.execution_time, 0)

    def test_complex_extraction_test(self):
        """Test complex extraction test method."""
        mock_extractor = MockExtractor("python")

        result = self.test_suite._test_complex_extraction("python", mock_extractor)

        self.assertIsInstance(result, TestResult)
        self.assertEqual(result.test_name, "complex_extraction")
        self.assertEqual(result.language, "python")

    def test_edge_cases_test(self):
        """Test edge cases test method."""
        mock_extractor = MockExtractor("python")

        result = self.test_suite._test_edge_cases("python", mock_extractor)

        self.assertIsInstance(result, TestResult)
        self.assertEqual(result.test_name, "edge_cases")
        self.assertEqual(result.language, "python")

    def test_error_handling_test(self):
        """Test error handling test method."""
        mock_extractor = MockExtractor("python")

        result = self.test_suite._test_error_handling("python", mock_extractor)

        self.assertIsInstance(result, TestResult)
        self.assertEqual(result.test_name, "error_handling")
        self.assertEqual(result.language, "python")
        self.assertGreaterEqual(result.accuracy_score, 0.0)
        self.assertLessEqual(result.accuracy_score, 1.0)

    def test_validation_test(self):
        """Test validation test method."""
        mock_extractor = MockExtractor("python")

        result = self.test_suite._test_validation("python", mock_extractor)

        self.assertIsInstance(result, TestResult)
        self.assertEqual(result.test_name, "validation")
        self.assertEqual(result.language, "python")

    def test_extractor_consistency(self):
        """Test extractor consistency testing."""
        self.test_suite.extractors = {
            "python": MockExtractor("python"),
            "javascript": MockExtractor("javascript"),
        }

        results = self.test_suite._test_extractor_consistency()

        self.assertIsInstance(results, list)
        for test in results:
            self.assertIn("pattern", test)
            self.assertIn("results", test)
            self.assertIn("passed", test)

    def test_multi_file_scenarios(self):
        """Test multi-file scenario testing."""
        self.test_suite.extractors = {"python": MockExtractor("python")}

        results = self.test_suite._test_multi_file_scenarios()

        self.assertIsInstance(results, list)
        if results:  # May be empty if no temporary files created
            for test in results:
                self.assertIn("test_name", test)

    def test_compare_extractor_performance(self):
        """Test extractor performance comparison."""
        self.test_suite.extractors = {
            "python": MockExtractor("python"),
            "javascript": MockExtractor("javascript"),
        }

        results = self.test_suite._compare_extractor_performance()

        self.assertIn("processing_speeds", results)
        self.assertIn("accuracy_scores", results)
        self.assertIn("resource_efficiency", results)
        self.assertIn("reliability_scores", results)

    def test_generate_performance_test_files(self):
        """Test performance test file generation."""
        self.test_suite.extractors = {"python": MockExtractor("python")}

        test_files = self.test_suite._generate_performance_test_files()

        self.assertIn("small_files", test_files)
        self.assertIn("medium_files", test_files)
        self.assertIn("large_files", test_files)

        # Check that files were generated for Python
        self.assertIn("python", test_files["small_files"])
        self.assertIn("python", test_files["medium_files"])
        self.assertIn("python", test_files["large_files"])

        # Check file sizes are increasing
        small_size = len(test_files["small_files"]["python"].encode("utf-8"))
        medium_size = len(test_files["medium_files"]["python"].encode("utf-8"))
        large_size = len(test_files["large_files"]["python"].encode("utf-8"))

        self.assertLess(small_size, medium_size)
        self.assertLess(medium_size, large_size)

    def test_benchmark_single_extractor(self):
        """Test single extractor benchmarking."""
        mock_extractor = MockExtractor("python")
        test_code = "print('hello world')"

        benchmark = self.test_suite._benchmark_single_extractor(
            mock_extractor,
            test_code,
            "test",
        )

        self.assertIn("test_name", benchmark)
        self.assertIn("code_size_bytes", benchmark)
        self.assertIn("execution_time", benchmark)
        self.assertIn("calls_found", benchmark)
        self.assertIn("throughput_kb_per_second", benchmark)

        self.assertEqual(benchmark["test_name"], "test")
        self.assertEqual(benchmark["code_size_bytes"], len(test_code.encode("utf-8")))
        self.assertGreaterEqual(benchmark["execution_time"], 0)

    def test_generate_comparative_analysis(self):
        """Test comparative analysis generation."""
        # Mock benchmark data
        language_benchmarks = {
            "python": {
                "small_files": [
                    {
                        "execution_time": 0.1,
                        "throughput_kb_per_second": 10,
                        "memory_peak_mb": 5,
                    },
                ],
                "medium_files": [
                    {
                        "execution_time": 0.2,
                        "throughput_kb_per_second": 8,
                        "memory_peak_mb": 10,
                    },
                ],
                "large_files": [
                    {
                        "execution_time": 0.5,
                        "throughput_kb_per_second": 6,
                        "memory_peak_mb": 20,
                    },
                ],
            },
            "javascript": {
                "small_files": [
                    {
                        "execution_time": 0.15,
                        "throughput_kb_per_second": 8,
                        "memory_peak_mb": 3,
                    },
                ],
                "medium_files": [
                    {
                        "execution_time": 0.25,
                        "throughput_kb_per_second": 6,
                        "memory_peak_mb": 8,
                    },
                ],
                "large_files": [
                    {
                        "execution_time": 0.6,
                        "throughput_kb_per_second": 4,
                        "memory_peak_mb": 15,
                    },
                ],
            },
        }

        analysis = self.test_suite._generate_comparative_analysis(language_benchmarks)

        self.assertIn("fastest_extractor", analysis)
        self.assertIn("most_memory_efficient", analysis)
        self.assertIn("performance_rankings", analysis)
        self.assertIn("recommendations", analysis)

    def test_check_performance_regressions(self):
        """Test performance regression checking."""
        # Mock benchmark data with regression
        language_benchmarks = {
            "python": {
                "small_files": [
                    {"execution_time": 10.0, "code_size_bytes": 1024},
                ],  # Very slow
                "medium_files": [],
                "large_files": [],
            },
        }

        regression_check = self.test_suite._check_performance_regressions(
            language_benchmarks,
        )

        self.assertIn("regressions_found", regression_check)
        self.assertIn("performance_status", regression_check)
        self.assertIn("recommendations", regression_check)

    def test_helper_methods(self):
        """Test various helper methods."""
        # Test _count_expected_complex_calls
        count = self.test_suite._count_expected_complex_calls("python")
        self.assertIsInstance(count, int)
        self.assertGreaterEqual(count, 3)

        # Test _get_edge_cases_for_language
        edge_cases = self.test_suite._get_edge_cases_for_language("python")
        self.assertIsInstance(edge_cases, list)
        self.assertGreater(len(edge_cases), 0)

        # Test _check_edge_case_coverage
        mock_call_sites = [
            CallSite(
                "test",
                1,
                0,
                0,
                10,
                "function",
                {"text_snippet": "lambda x: x"},
                "python",
                Path("test.py"),
            ),
        ]
        coverage = self.test_suite._check_edge_case_coverage(
            mock_call_sites,
            "lambda_calls",
        )
        self.assertIsInstance(coverage, bool)

        # Test _generate_performance_recommendations
        mock_metrics = {
            "python": {"avg_execution_time": 0.1, "avg_memory_usage": 50},
            "javascript": {"avg_execution_time": 0.05, "avg_memory_usage": 30},
        }
        recommendations = self.test_suite._generate_performance_recommendations(
            mock_metrics,
        )
        self.assertIsInstance(recommendations, list)
        self.assertGreater(len(recommendations), 0)


class TestIntegrationTester(unittest.TestCase):
    """Test cases for IntegrationTester class."""

    def setUp(self):
        """Set up test fixtures."""
        self.integration_tester = IntegrationTester()

    def test_initialization(self):
        """Test IntegrationTester initialization."""
        self.assertIsInstance(self.integration_tester.test_suite, ExtractorTestSuite)

    def test_test_complete_workflow(self):
        """Test complete workflow testing."""
        # Mock the test suite for faster testing
        self.integration_tester.test_suite.extractors = {
            "python": MockExtractor("python"),
        }

        results = self.integration_tester.test_complete_workflow()

        self.assertIn("workflow_stages", results)
        self.assertIn("end_to_end_success", results)
        self.assertIn("total_time", results)
        self.assertIn("data_integrity_check", results)
        self.assertIn("error_recovery_test", results)

    def test_validate_accuracy(self):
        """Test accuracy validation."""
        # Mock the test suite
        self.integration_tester.test_suite.extractors = {
            "python": MockExtractor("python"),
        }

        results = self.integration_tester.validate_accuracy()

        self.assertIn("overall_accuracy", results)
        self.assertIn("language_accuracy", results)
        self.assertIn("accuracy_test_results", results)
        self.assertIn("validation_methodology", results)

    def test_test_error_handling(self):
        """Test error handling testing."""
        # Mock the test suite
        self.integration_tester.test_suite.extractors = {
            "python": MockExtractor("python"),
        }

        results = self.integration_tester.test_error_handling()

        self.assertIn("robustness_score", results)
        self.assertIn("language_error_handling", results)
        self.assertIn("critical_failures", results)
        self.assertIn("recovery_mechanisms", results)
        self.assertIn("error_categories_tested", results)

    def test_test_performance_integration(self):
        """Test performance integration testing."""
        results = self.integration_tester.test_performance_integration()

        self.assertIn("concurrent_processing", results)
        self.assertIn("memory_efficiency", results)
        self.assertIn("scalability_analysis", results)
        self.assertIn("performance_degradation_check", results)
        self.assertIn("resource_utilization", results)

    def test_workflow_stage_methods(self):
        """Test individual workflow stage methods."""
        # Mock the test suite
        self.integration_tester.test_suite.extractors = {
            "python": MockExtractor("python"),
        }

        # Test extractor initialization stage
        init_result = self.integration_tester._test_extractor_initialization()
        self.assertIn("stage_name", init_result)
        self.assertIn("passed", init_result)
        self.assertIn("extractors_initialized", init_result)

        # Test extraction workflow stage
        workflow_result = self.integration_tester._test_extraction_workflow()
        self.assertIn("stage_name", workflow_result)
        self.assertIn("passed", workflow_result)
        self.assertIn("successful_extractions", workflow_result)

        # Test result aggregation stage
        aggregation_result = self.integration_tester._test_result_aggregation()
        self.assertIn("stage_name", aggregation_result)
        self.assertIn("passed", aggregation_result)
        self.assertIn("aggregation_tests", aggregation_result)

        # Test cleanup workflow stage
        cleanup_result = self.integration_tester._test_cleanup_workflow()
        self.assertIn("stage_name", cleanup_result)
        self.assertIn("passed", cleanup_result)
        self.assertIn("cleanup_tests", cleanup_result)

    def test_data_integrity(self):
        """Test data integrity testing."""
        integrity_result = self.integration_tester._test_data_integrity()

        self.assertIn("position_accuracy", integrity_result)
        self.assertIn("function_name_accuracy", integrity_result)
        self.assertIn("call_type_consistency", integrity_result)
        self.assertIn("context_preservation", integrity_result)

    def test_error_recovery(self):
        """Test error recovery testing."""
        # Mock the test suite
        self.integration_tester.test_suite.extractors = {
            "python": MockExtractor("python"),
        }

        recovery_result = self.integration_tester._test_error_recovery()

        self.assertIn("graceful_degradation", recovery_result)
        self.assertIn("partial_success_handling", recovery_result)
        self.assertIn("error_propagation", recovery_result)

    def test_validate_language_accuracy(self):
        """Test language accuracy validation."""
        mock_extractor = MockExtractor("python")

        accuracy_result = self.integration_tester._validate_language_accuracy(
            "python",
            mock_extractor,
        )

        self.assertIn("overall_score", accuracy_result)
        self.assertIn("position_accuracy", accuracy_result)
        self.assertIn("name_accuracy", accuracy_result)
        self.assertIn("type_accuracy", accuracy_result)
        self.assertIn("context_completeness", accuracy_result)
        self.assertIn("test_details", accuracy_result)

        self.assertGreaterEqual(accuracy_result["overall_score"], 0.0)
        self.assertLessEqual(accuracy_result["overall_score"], 1.0)

    def test_recovery_mechanisms(self):
        """Test recovery mechanisms testing."""
        # Mock the test suite
        self.integration_tester.test_suite.extractors = {
            "python": MockExtractor("python"),
        }

        recovery_test = self.integration_tester._test_recovery_mechanisms()

        self.assertIn("fallback_strategies", recovery_test)
        self.assertIn("partial_processing", recovery_test)
        self.assertIn("error_isolation", recovery_test)


class TestUtilityFunctions(unittest.TestCase):
    """Test cases for utility functions."""

    def test_run_complete_extractor_test_suite(self):
        """Test the main test suite runner."""
        # This is a comprehensive test, so we'll do a basic check
        with patch(
            "chunker.extractors.testing.integration_tester.ExtractorTestSuite",
        ) as mock_suite_class:
            with patch(
                "chunker.extractors.testing.integration_tester.IntegrationTester",
            ) as mock_tester_class:
                # Mock the classes to return simple results
                mock_suite = Mock()
                mock_suite.run_all_tests.return_value = {
                    "overall_passed": True,
                    "total_tests": 10,
                    "passed_tests": 10,
                }
                mock_suite_class.return_value = mock_suite

                mock_tester = Mock()
                mock_tester.test_complete_workflow.return_value = {
                    "end_to_end_success": True,
                }
                mock_tester.validate_accuracy.return_value = {"overall_accuracy": 0.9}
                mock_tester.test_error_handling.return_value = {"robustness_score": 0.8}
                mock_tester.test_performance_integration.return_value = {
                    "concurrent_processing": {},
                }
                mock_tester_class.return_value = mock_tester

                results = run_complete_extractor_test_suite()

                self.assertIn("test_suite_version", results)
                self.assertIn("execution_timestamp", results)
                self.assertIn("overall_status", results)
                self.assertIn("component_results", results)
                self.assertIn("summary", results)
                self.assertIn("recommendations", results)
                self.assertIn("total_execution_time", results)

    def test_generate_overall_summary(self):
        """Test overall summary generation."""
        mock_component_results = {
            "test_suite": {
                "overall_passed": True,
                "total_tests": 20,
                "passed_tests": 18,
                "performance_summary": {"grade": "A"},
            },
            "accuracy_validation": {
                "overall_accuracy": 0.85,
                "language_accuracy": {"python": {"overall_score": 0.9}},
            },
            "workflow_tests": {"end_to_end_success": False},
        }

        summary = _generate_overall_summary(mock_component_results)

        self.assertIn("total_tests_run", summary)
        self.assertIn("total_tests_passed", summary)
        self.assertIn("overall_success_rate", summary)
        self.assertIn("component_status", summary)
        self.assertIn("critical_issues", summary)
        self.assertIn("performance_summary", summary)
        self.assertIn("accuracy_summary", summary)

        self.assertEqual(summary["total_tests_run"], 20)
        self.assertEqual(summary["total_tests_passed"], 18)
        self.assertEqual(summary["overall_success_rate"], 0.9)
        self.assertEqual(summary["component_status"]["test_suite"], "PASSED")
        self.assertEqual(summary["component_status"]["workflow_tests"], "FAILED")

    def test_generate_overall_recommendations(self):
        """Test overall recommendations generation."""
        mock_component_results = {
            "test_suite": {
                "overall_passed": False,
                "summary": {"overall_success_rate": 0.6},
            },
            "accuracy_validation": {
                "overall_accuracy": 0.5,
                "language_accuracy": {
                    "python": {"overall_score": 0.4},
                    "javascript": {"overall_score": 0.6},
                },
            },
            "error_handling": {
                "robustness_score": 0.7,
                "critical_failures": [{"language": "python", "issue": "crash"}],
            },
            "performance_integration": {
                "memory_efficiency": {
                    "memory_cleanup": {"python": 60, "javascript": 30},
                },
            },
        }

        recommendations = _generate_overall_recommendations(mock_component_results)

        self.assertIsInstance(recommendations, list)
        self.assertGreater(len(recommendations), 0)

        # Check that appropriate recommendations are generated
        rec_text = " ".join(recommendations)
        self.assertIn("production deployment", rec_text.lower())


class TestErrorHandling(unittest.TestCase):
    """Test error handling in integration testing."""

    def test_extractor_failure_handling(self):
        """Test handling of extractor failures."""
        # Create a test suite with a failing extractor
        test_suite = ExtractorTestSuite()
        test_suite.extractors = {"python": MockExtractor("python", should_fail=True)}

        # Test that failures are handled gracefully
        results = test_suite.test_language_extractors()

        self.assertIn("python", results)
        python_results = results["python"]

        # Should still return results structure even with failures
        self.assertIn("extractor_class", python_results)
        self.assertIn("tests", python_results)
        self.assertIn("overall_passed", python_results)
        self.assertFalse(
            python_results["overall_passed"],
        )  # Should fail due to mock failure

    def test_missing_extractor_handling(self):
        """Test handling of missing extractors."""
        test_suite = ExtractorTestSuite()
        test_suite.extractors = {}  # No extractors

        results = test_suite.test_language_extractors()

        # Should return empty results without crashing
        self.assertIsInstance(results, dict)
        self.assertEqual(len(results), 0)

    def test_invalid_data_handling(self):
        """Test handling of invalid data in utility functions."""
        # Test with None input
        summary = _generate_overall_summary(None)
        self.assertIn("summary_generation_error", summary)

        recommendations = _generate_overall_recommendations(None)
        self.assertIsInstance(recommendations, list)
        self.assertGreater(len(recommendations), 0)  # Should have error message

    def test_test_result_error_handling(self):
        """Test TestResult error handling."""
        result = TestResult("test", "python")

        # Test with None exception
        result.add_error("Error", None)
        self.assertEqual(len(result.errors), 1)
        self.assertIn("Error", result.errors[0])

        # Test with complex exception
        try:
            raise ValueError("Complex error") from Exception("Root cause")
        except Exception as e:
            result.add_error("Nested error", e)

        self.assertEqual(len(result.errors), 2)
        self.assertIn("Nested error", result.errors[1])


if __name__ == "__main__":
    unittest.main()



"""Tests for workflow validation utilities"""

import tempfile
from pathlib import Path

import yaml

from chunker.cicd.workflow_validator import WorkflowValidator, validate_all_workflows


class TestWorkflowValidator:
    """Test the workflow validator"""

    def setup_method(self):
        """Set up test instance"""
        self.validator = WorkflowValidator()

    def test_validate_valid_workflow(self):
        """Test validation of a complete valid workflow"""
        workflow = {
            "name": "CI",
            "on": ["push", "pull_request"],
            "jobs": {
                "test": {
                    "runs-on": "ubuntu-latest",
                    "steps": [
                        {"uses": "actions/checkout@v3"},
                        {"name": "Run tests", "run": "pytest"},
                    ],
                },
            },
        }
        with tempfile.NamedTemporaryFile(
            encoding="utf-8",
            mode="w",
            suffix=".yml",
            delete=False,
        ) as f:
            yaml.dump(workflow, f)
            workflow_path = Path(f.name)
        try:
            is_valid, errors, _warnings = self.validator.validate_file(workflow_path)
            assert is_valid is True
            assert len(errors) == 0
        finally:
            workflow_path.unlink()

    def test_validate_complex_workflow(self):
        """Test validation of complex workflow with matrix"""
        workflow = {
            "name": "Build",
            "on": {
                "push": {"branches": ["main", "develop"], "tags": ["v*"]},
                "pull_request": {"types": ["opened", "synchronize"]},
            },
            "env": {"PYTHON_VERSION": "3.10"},
            "jobs": {
                "test": {
                    "runs-on": "${{ matrix.os }}",
                    "strategy": {
                        "matrix": {
                            "os": ["ubuntu-latest", "windows-latest"],
                            "python": ["3.8", "3.9", "3.10"],
                        },
                    },
                    "steps": [
                        {"uses": "actions/checkout@v3"},
                        {
                            "uses": "actions/setup-python@v4",
                            "with": {"python-version": "${{ matrix.python }}"},
                        },
                        {"run": "python -m pytest"},
                    ],
                },
            },
        }
        with tempfile.NamedTemporaryFile(
            encoding="utf-8",
            mode="w",
            suffix=".yml",
            delete=False,
        ) as f:
            yaml.dump(workflow, f)
            workflow_path = Path(f.name)
        try:
            is_valid, errors, _warnings = self.validator.validate_file(workflow_path)
            assert is_valid is True
            assert len(errors) == 0
        finally:
            workflow_path.unlink()

    def test_validate_missing_name(self):
        """Test validation catches missing name"""
        workflow = {
            "on": "push",
            "jobs": {
                "test": {"runs-on": "ubuntu-latest", "steps": [{"run": "echo test"}]},
            },
        }
        with tempfile.NamedTemporaryFile(
            encoding="utf-8",
            mode="w",
            suffix=".yml",
            delete=False,
        ) as f:
            yaml.dump(workflow, f)
            workflow_path = Path(f.name)
        try:
            is_valid, errors, _warnings = self.validator.validate_file(workflow_path)
            assert is_valid is False
            assert any("Missing required field: 'name'" in e for e in errors)
        finally:
            workflow_path.unlink()

    def test_validate_invalid_job_id(self):
        """Test validation catches invalid job IDs"""
        workflow = {
            "name": "Test",
            "on": "push",
            "jobs": {
                "123-invalid": {
                    "runs-on": "ubuntu-latest",
                    "steps": [{"run": "echo test"}],
                },
            },
        }
        with tempfile.NamedTemporaryFile(
            encoding="utf-8",
            mode="w",
            suffix=".yml",
            delete=False,
        ) as f:
            yaml.dump(workflow, f)
            workflow_path = Path(f.name)
        try:
            is_valid, errors, _warnings = self.validator.validate_file(workflow_path)
            assert is_valid is False
            assert any("Invalid job ID" in e for e in errors)
        finally:
            workflow_path.unlink()

    def test_validate_unknown_runner(self):
        """Test validation warns about unknown runners"""
        workflow = {
            "name": "Test",
            "on": "push",
            "jobs": {
                "test": {"runs-on": "custom-runner", "steps": [{"run": "echo test"}]},
            },
        }
        with tempfile.NamedTemporaryFile(
            encoding="utf-8",
            mode="w",
            suffix=".yml",
            delete=False,
        ) as f:
            yaml.dump(workflow, f)
            workflow_path = Path(f.name)
        try:
            is_valid, _errors, warnings = self.validator.validate_file(workflow_path)
            assert is_valid is True
            assert any("unknown runner" in w for w in warnings)
        finally:
            workflow_path.unlink()

    def test_validate_self_hosted_runner(self):
        """Test validation accepts self-hosted runners"""
        workflow = {
            "name": "Test",
            "on": "push",
            "jobs": {
                "test": {"runs-on": "self-hosted", "steps": [{"run": "echo test"}]},
            },
        }
        with tempfile.NamedTemporaryFile(
            encoding="utf-8",
            mode="w",
            suffix=".yml",
            delete=False,
        ) as f:
            yaml.dump(workflow, f)
            workflow_path = Path(f.name)
        try:
            is_valid, _errors, warnings = self.validator.validate_file(workflow_path)
            assert is_valid is True
            assert not any("unknown runner" in w for w in warnings)
        finally:
            workflow_path.unlink()

    def test_validate_schedule_cron(self):
        """Test validation of schedule with cron"""
        workflow = {
            "name": "Scheduled",
            "on": {"schedule": [{"cron": "0 0 * * *"}]},
            "jobs": {
                "nightly": {
                    "runs-on": "ubuntu-latest",
                    "steps": [{"run": 'echo "Nightly build"'}],
                },
            },
        }
        with tempfile.NamedTemporaryFile(
            encoding="utf-8",
            mode="w",
            suffix=".yml",
            delete=False,
        ) as f:
            yaml.dump(workflow, f)
            workflow_path = Path(f.name)
        try:
            is_valid, errors, _warnings = self.validator.validate_file(workflow_path)
            assert is_valid is True
            assert len(errors) == 0
        finally:
            workflow_path.unlink()

    def test_validate_permissions(self):
        """Test validation of permissions"""
        workflow = {
            "name": "Deploy",
            "on": "push",
            "permissions": {
                "contents": "read",
                "packages": "write",
                "invalid-scope": "read",
            },
            "jobs": {
                "deploy": {
                    "runs-on": "ubuntu-latest",
                    "steps": [{"run": "echo deploy"}],
                },
            },
        }
        with tempfile.NamedTemporaryFile(
            encoding="utf-8",
            mode="w",
            suffix=".yml",
            delete=False,
        ) as f:
            yaml.dump(workflow, f)
            workflow_path = Path(f.name)
        try:
            is_valid, _errors, warnings = self.validator.validate_file(workflow_path)
            assert is_valid is True
            assert any("Unknown permission scope" in w for w in warnings)
        finally:
            workflow_path.unlink()

    def test_validate_action_without_version(self):
        """Test validation warns about actions without versions"""
        workflow = {
            "name": "Test",
            "on": "push",
            "jobs": {
                "test": {
                    "runs-on": "ubuntu-latest",
                    "steps": [
                        {"uses": "actions/checkout"},
                        {"uses": "actions/setup-python@v4"},
                    ],
                },
            },
        }
        with tempfile.NamedTemporaryFile(
            encoding="utf-8",
            mode="w",
            suffix=".yml",
            delete=False,
        ) as f:
            yaml.dump(workflow, f)
            workflow_path = Path(f.name)
        try:
            is_valid, _errors, warnings = self.validator.validate_file(workflow_path)
            assert is_valid is True
            assert any("should specify action version" in w for w in warnings)
        finally:
            workflow_path.unlink()

    def test_validate_step_with_both_uses_and_run(self):
        """Test validation warns about steps with both uses and run"""
        workflow = {
            "name": "Test",
            "on": "push",
            "jobs": {
                "test": {
                    "runs-on": "ubuntu-latest",
                    "steps": [
                        {"uses": "actions/checkout@v3", "run": 'echo "This is wrong"'},
                    ],
                },
            },
        }
        with tempfile.NamedTemporaryFile(
            encoding="utf-8",
            mode="w",
            suffix=".yml",
            delete=False,
        ) as f:
            yaml.dump(workflow, f)
            workflow_path = Path(f.name)
        try:
            is_valid, _errors, warnings = self.validator.validate_file(workflow_path)
            assert is_valid is True
            assert any("has both 'uses' and 'run'" in w for w in warnings)
        finally:
            workflow_path.unlink()


class TestValidateAllWorkflows:
    """Test the validate_all_workflows function"""

    @classmethod
    def test_validate_directory(cls):
        """Test validating all workflows in a directory"""
        with tempfile.TemporaryDirectory() as tmpdir:
            workflows_dir = Path(tmpdir) / ".github" / "workflows"
            workflows_dir.mkdir(parents=True)
            valid_workflow = {
                "name": "Valid",
                "on": "push",
                "jobs": {
                    "test": {
                        "runs-on": "ubuntu-latest",
                        "steps": [{"run": "echo valid"}],
                    },
                },
            }
            with Path(workflows_dir / "valid.yml").open(
                "w",
                encoding="utf-8",
            ) as f:
                yaml.dump(valid_workflow, f)
            invalid_workflow = {
                "name": "Invalid",
                "jobs": {"test": {"steps": [{"run": "echo invalid"}]}},
            }
            with Path(workflows_dir / "invalid.yaml").open("w", encoding="utf-8") as f:
                yaml.dump(invalid_workflow, f)
            results = validate_all_workflows(workflows_dir)
            assert "valid.yml" in results
            assert "invalid.yaml" in results
            valid_is_valid, valid_errors, _valid_warnings = results["valid.yml"]
            assert valid_is_valid is True
            assert len(valid_errors) == 0
            invalid_is_valid, invalid_errors, _invalid_warnings = results[
                "invalid.yaml"
            ]
            assert invalid_is_valid is False
            assert len(invalid_errors) > 0

    @classmethod
    def test_validate_nonexistent_directory(cls):
        """Test validating nonexistent directory returns empty results"""
        results = validate_all_workflows(Path("/nonexistent/workflows"))
        assert results == {}
